#!/bin/sh
# Syd: rock-solid application kernel
# data/syd.sh: Defines 'esyd' command, the multi functional syd helper.
#
# esyd is written in portable shell.
# It should work fine with POSIX sh, Bash and Zsh.
# If you spot a problem running this with either of them,
# please report a bug at: https://todo.sr.ht/~alip/syd
#
# Copyright (c) 2023, 2024 Ali Polatel <alip@chesswob.org>
# SPDX-License-Identifier: GPL-3.0

esyd() {
    local cmd="${1}"

    shift
    case "${cmd}" in
    api)
        echo -n 3
        ;;
    check)
        syd-chk
        local r=$?
        if [ -t 1 ]; then
            case $r in
            0)
                echo >&2 "Running under syd."
                ;;
            *)
                echo >&2 "Not running under syd."
                ;;
            esac
        fi
        return $r
        ;;
    panic|reset|stat)
        [ -c /dev/syd/${cmd} ]
        ;;
    load)
        if [ ${#} -ne 1 ]; then
            echo >&2 "esyd: ${cmd} takes exactly one extra argument"
            return 1
        fi
        [ -c "/dev/syd/load/${1}" ]
        ;;
    lock)
        [ -c '/dev/syd/lock:on' ]
        ;;
    unlock)
        [ -c '/dev/syd/lock:off' ]
        ;;
    exec_lock)
        [ -c '/dev/syd/lock:exec' ]
        ;;
    info)
        if [ -c /dev/syd ]; then
            if command -v jq >/dev/null 2>&1; then
                local out
                out=$(mktemp)
                if [ -n "$BASH_VERSION" ]; then
                    # We're in Bash
                    # This works with lock:exec
                    IFS=$'\n' readarray -t syd < /dev/syd
                    echo "${syd[*]}" > "${out}"
                elif [ -n "$ZSH_VERSION" ]; then
                    syd=$(</dev/syd)
                    echo "${syd}" > "${out}"
                else
                    # Fallback for POSIX sh
                    # This needs lock:off, doesn't work with lock:exec
                    cat /dev/syd > "${out}"
                fi

                jq "${@}" < "${out}"
                r=$?
                rm -f "${out}"
                return $r
            else
                if [ -n "$BASH_VERSION" ]; then
                    # We're in Bash
                    # This works with lock:exec
                    IFS=$'\n' readarray -t syd < /dev/syd
                    echo "${syd[*]}"
                elif [ -n "$ZSH_VERSION" ]; then
                    syd=$(</dev/syd)
                    echo "${syd}"
                else
                    # Fallback for POSIX sh
                    # This needs lock:off, doesn't work with lock:exec
                    cat /dev/syd
                fi
            fi
        else
            return 1
        fi
        ;;
    mem_max)
        if [ ${#} -ne 1 ]; then
            echo >&2 "esyd: ${cmd} takes exactly one extra argument"
            return 1
        fi
        [ -c "/dev/syd/mem/max:${1}" ]
        ;;
    vm_max)
        if [ ${#} -ne 1 ]; then
            echo >&2 "esyd: ${cmd} takes exactly one extra argument"
            return 1
        fi
        [ -c "/dev/syd/mem/vm_max:${1}" ]
        ;;
    kill_mem)
        [ -c '/dev/syd/mem/kill:1' ]
        ;;
    nokill_mem)
        [ -c '/dev/syd/mem/kill:0' ]
        ;;
    filter_mem)
        [ -c '/dev/syd/filter/mem:1' ]
        ;;
    unfilter_mem)
        [ -c '/dev/syd/filter/mem:0' ]
        ;;
    pid_max)
        if [ ${#} -ne 1 ]; then
            echo >&2 "esyd: ${cmd} takes exactly one extra argument"
            return 1
        fi
        [ -c "/dev/syd/pid/max:${1}" ]
        ;;
    kill_pid)
        [ -c '/dev/syd/pid/kill:1' ]
        ;;
    nokill_pid)
        [ -c '/dev/syd/pid/kill:0' ]
        ;;
    filter_pid)
        [ -c '/dev/syd/filter/pid:1' ]
        ;;
    unfilter_pid)
        [ -c '/dev/syd/filter/pid:0' ]
        ;;
    exec)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        [ -c "$(syd-exec ${@})" ]
        ;;
    force)
        if [ ${#} -ne 3 ]; then
            echo >&2 "esyd: ${cmd} takes exactly three arguments"
            return 1
        fi
        _esyd_path 'force' '+' "${1}:${2}:${3}"
        ;;
    enabled|enabled_path)
        for capability in read stat write create delete truncate attr chown chgrp ioctl node tmpfile; do
            test -c "/dev/syd/sandbox/${capability}?" || return 1
        done
        ;;
    enable|enable_path)
        for capability in read stat write create delete truncate attr chown chgrp ioctl node tmpfile; do
            test -c "/dev/syd/sandbox/${capability}:on" || return 1
        done
        ;;
    disable|disable_path)
        for capability in read stat write create delete truncate attr chown chgrp ioctl node tmpfile; do
            test -c "/dev/syd/sandbox/${capability}:off" || return 1
        done
        ;;
    enabled_mem)
        [ -c '/dev/syd/sandbox/mem?' ]
        ;;
    enable_mem)
        [ -c '/dev/syd/sandbox/mem:on' ]
        ;;
    disable_mem)
        [ -c '/dev/syd/sandbox/mem:off' ]
        ;;
    enabled_pid)
        [ -c '/dev/syd/sandbox/pid?' ]
        ;;
    enable_pid)
        [ -c '/dev/syd/sandbox/pid:on' ]
        ;;
    disable_pid)
        [ -c '/dev/syd/sandbox/pid:off' ]
        ;;
    enabled_read)
        [ -c '/dev/syd/sandbox/read?' ]
        ;;
    enable_read)
        [ -c '/dev/syd/sandbox/read:on' ]
        ;;
    disable_read)
        [ -c '/dev/syd/sandbox/read:off' ]
        ;;
    enabled_stat)
        [ -c '/dev/syd/sandbox/stat?' ]
        ;;
    enable_stat)
        [ -c '/dev/syd/sandbox/stat:on' ]
        ;;
    disable_stat)
        [ -c '/dev/syd/sandbox/stat:off' ]
        ;;
    enabled_write)
        [ -c '/dev/syd/sandbox/write?' ]
        ;;
    enable_write)
        [ -c '/dev/syd/sandbox/write:on' ]
        ;;
    disable_write)
        [ -c '/dev/syd/sandbox/write:off' ]
        ;;
    enabled_create)
        [ -c '/dev/syd/sandbox/create?' ]
        ;;
    enable_create)
        [ -c '/dev/syd/sandbox/create:on' ]
        ;;
    disable_create)
        [ -c '/dev/syd/sandbox/create:off' ]
        ;;
    enabled_delete)
        [ -c '/dev/syd/sandbox/delete?' ]
        ;;
    enable_delete)
        [ -c '/dev/syd/sandbox/delete:on' ]
        ;;
    disable_delete)
        [ -c '/dev/syd/sandbox/delete:off' ]
        ;;
    enabled_truncate)
        [ -c '/dev/syd/sandbox/truncate?' ]
        ;;
    enable_truncate)
        [ -c '/dev/syd/sandbox/truncate:on' ]
        ;;
    disable_truncate)
        [ -c '/dev/syd/sandbox/truncate:off' ]
        ;;
    enabled_attr)
        [ -c '/dev/syd/sandbox/attr?' ]
        ;;
    enable_attr)
        [ -c '/dev/syd/sandbox/attr:on' ]
        ;;
    disable_attr)
        [ -c '/dev/syd/sandbox/attr:off' ]
        ;;
    enabled_ioctl)
        [ -c '/dev/syd/sandbox/ioctl?' ]
        ;;
    enable_ioctl)
        [ -c '/dev/syd/sandbox/ioctl:on' ]
        ;;
    disable_ioctl)
        [ -c '/dev/syd/sandbox/ioctl:off' ]
        ;;
    enabled_chown)
        [ -c '/dev/syd/sandbox/chown?' ]
        ;;
    enable_chown)
        [ -c '/dev/syd/sandbox/chown:on' ]
        ;;
    disable_chown)
        [ -c '/dev/syd/sandbox/chown:off' ]
        ;;
    enabled_chgrp)
        [ -c '/dev/syd/sandbox/chgrp?' ]
        ;;
    enable_chgrp)
        [ -c '/dev/syd/sandbox/chgrp:on' ]
        ;;
    disable_chgrp)
        [ -c '/dev/syd/sandbox/chgrp:off' ]
        ;;
    enabled_node)
        [ -c '/dev/syd/sandbox/node?' ]
        ;;
    enable_node)
        [ -c '/dev/syd/sandbox/node:on' ]
        ;;
    disable_node)
        [ -c '/dev/syd/sandbox/node:off' ]
        ;;
    enabled_tmpfile)
        [ -c '/dev/syd/sandbox/tmpfile?' ]
        ;;
    enable_tmpfile)
        [ -c '/dev/syd/sandbox/tmpfile:on' ]
        ;;
    disable_tmpfile)
        [ -c '/dev/syd/sandbox/tmpfile:off' ]
        ;;
    enabled_exec)
        [ -c '/dev/syd/sandbox/exec?' ]
        ;;
    enable_exec)
        [ -c '/dev/syd/sandbox/exec:on' ]
        ;;
    disable_exec)
        [ -c '/dev/syd/sandbox/exec:off' ]
        ;;
    enabled_force)
        [ -c '/dev/syd/sandbox/force?' ]
        ;;
    enable_force)
        [ -c '/dev/syd/sandbox/force:on' ]
        ;;
    disable_force)
        [ -c '/dev/syd/sandbox/force:off' ]
        ;;
    enabled_net)
        [ -c '/dev/syd/sandbox/net?' ]
        ;;
    enable_net)
        [ -c '/dev/syd/sandbox/net:on' ]
        ;;
    disable_net)
        [ -c '/dev/syd/sandbox/net:off' ]
        ;;
    allow|allow_path)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        # allow is shorthand for all glob capabilities.
        for capability in read stat write create delete truncate attr chown chgrp ioctl node tmpfile; do
            _esyd_path "allow/${capability}" '+' "${@}"
        done
        ;;
    disallow|disallow_path)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        # disallow is shorthand for all glob capabilities.
        for capability in read stat write create delete truncate attr chown chgrp ioctl node tmpfile; do
            _esyd_path "allow/${capability}" "${op}" "${@}"
        done
        ;;
    deny|deny_path)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        # deny is shorthand for all glob capabilities.
        for capability in read stat write create delete truncate attr chown chgrp ioctl node tmpfile; do
            _esyd_path "deny/${capability}" '+' "${@}"
        done
        ;;
    nodeny|nodeny_path)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        # nodeny is shorthand for all glob capabilities.
        for capability in read stat write create delete truncate attr chown chgrp ioctl node tmpfile; do
            _esyd_path "deny/${capability}" "${op}" "${@}"
        done
        ;;
    allow_read)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/read' '+' "${@}"
        ;;
    disallow_read)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/read' "${op}" "${@}"
        ;;
    deny_read)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/read' '+' "${@}"
        ;;
    nodeny_read)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/read' "${op}" "${@}"
        ;;
    allow_stat)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/stat' '+' "${@}"
        ;;
    disallow_stat)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/stat' "${op}" "${@}"
        ;;
    deny_stat)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/stat' '+' "${@}"
        ;;
    nodeny_stat)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/stat' "${op}" "${@}"
        ;;
    allow_write)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/write' '+' "${@}"
        ;;
    disallow_write)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/write' "${op}" "${@}"
        ;;
    deny_write)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/write' '+' "${@}"
        ;;
    nodeny_write)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/write' "${op}" "${@}"
        ;;
    allow_create)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/create' '+' "${@}"
        ;;
    disallow_create)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/create' "${op}" "${@}"
        ;;
    deny_create)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/create' '+' "${@}"
        ;;
    nodeny_create)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/create' "${op}" "${@}"
        ;;
    allow_delete)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/delete' '+' "${@}"
        ;;
    disallow_delete)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/delete' "${op}" "${@}"
        ;;
    deny_delete)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/delete' '+' "${@}"
        ;;
    nodeny_delete)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/delete' "${op}" "${@}"
        ;;
    allow_truncate)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/truncate' '+' "${@}"
        ;;
    disallow_truncate)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/truncate' "${op}" "${@}"
        ;;
    deny_truncate)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/truncate' '+' "${@}"
        ;;
    nodeny_truncate)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/truncate' "${op}" "${@}"
        ;;
    allow_attr)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/attr' '+' "${@}"
        ;;
    disallow_attr)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/attr' "${op}" "${@}"
        ;;
    deny_attr)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/attr' '+' "${@}"
        ;;
    nodeny_attr)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/attr' "${op}" "${@}"
        ;;
    allow_chown)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/chown' '+' "${@}"
        ;;
    disallow_chown)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/chown' "${op}" "${@}"
        ;;
    deny_chown)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/chown' '+' "${@}"
        ;;
    nodeny_chown)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/chown' "${op}" "${@}"
        ;;
    allow_chgrp)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/chgrp' '+' "${@}"
        ;;
    disallow_chgrp)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/chgrp' "${op}" "${@}"
        ;;
    deny_chgrp)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/chgrp' '+' "${@}"
        ;;
    nodeny_chgrp)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/chgrp' "${op}" "${@}"
        ;;
    allow_ioctl)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/ioctl' '+' "${@}"
        ;;
    disallow_ioctl)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/ioctl' "${op}" "${@}"
        ;;
    deny_ioctl)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/ioctl' '+' "${@}"
        ;;
    nodeny_ioctl)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/ioctl' "${op}" "${@}"
        ;;
    allow_node)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/node' '+' "${@}"
        ;;
    disallow_node)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/node' "${op}" "${@}"
        ;;
    deny_node)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/node' '+' "${@}"
        ;;
    nodeny_node)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/node' "${op}" "${@}"
        ;;
    allow_tmpfile)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/tmpfile' '+' "${@}"
        ;;
    disallow_tmpfile)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/tmpfile' "${op}" "${@}"
        ;;
    deny_tmpfile)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/tmpfile' '+' "${@}"
        ;;
    nodeny_tmpfile)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/tmpfile' "${op}" "${@}"
        ;;
    allow_exec)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/exec' '+' "${@}"
        ;;
    disallow_exec)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'allow/exec' "${op}" "${@}"
        ;;
    deny_exec)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/exec' '+' "${@}"
        ;;
    nodeny_exec)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'deny/exec' "${op}" "${@}"
        ;;
    allow_net)
        local op='-'
        local c='allow/net/bind'
        [ "${1}" == '--all' ] && op='^' && shift
        case "${1}" in
        '--connect')
            c='allow/net/connect'
            shift;;
        '--send')
            c='allow/net/send'
            shift;;
        esac
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_net "${c}" '+' "${@}"
        ;;
    disallow_net)
        local op='-'
        local c='allow/net/bind'
        [ "${1}" == '--all' ] && op='^' && shift
        case "${1}" in
        '--connect')
            c='allow/net/connect'
            shift;;
        '--send')
            c='allow/net/send'
            shift;;
        esac
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_net "${c}" "${op}" "${@}"
        ;;
    deny_net)
        local op='-'
        local c='deny/net/bind'
        [ "${1}" == '--all' ] && op='^' && shift
        case "${1}" in
        '--connect')
            c='allow/net/connect'
            shift;;
        '--send')
            c='allow/net/send'
            shift;;
        esac
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_net "${c}" '+' "${@}"
        ;;
    nodeny_net)
        local op='-'
        local c='deny/net/bind'
        [ "${1}" == '--all' ] && op='^' && shift
        case "${1}" in
        '--connect')
            c='allow/net/connect'
            shift;;
        '--send')
            c='allow/net/send'
            shift;;
        esac
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_net "${c}" "${op}" "${@}"
        ;;
    addfilter|addfilter_path)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        for capability in write create delete truncate attr chown chgrp ioctl node tmpfile; do
            _esyd_path "filter/${capability}" '+' "${@}" || return 1
        done
        ;;
    rmfilter|rmfilter_path)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        for capability in write create delete truncate attr chown chgrp ioctl node tmpfile; do
            _esyd_path "filter/${capability}" "${op}" "${@}" || return 1
        done
        ;;
    addfilter_read)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/read' '+' "${@}"
        ;;
    rmfilter_read)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/read' "${op}" "${@}"
        ;;
    addfilter_stat)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/stat' '+' "${@}"
        ;;
    rmfilter_stat)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/stat' "${op}" "${@}"
        ;;
    addfilter_write)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/write' '+' "${@}"
        ;;
    rmfilter_write)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/write' "${op}" "${@}"
        ;;
    addfilter_create)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/create' '+' "${@}"
        ;;
    rmfilter_create)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/create' "${op}" "${@}"
        ;;
    addfilter_delete)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/delete' '+' "${@}"
        ;;
    rmfilter_delete)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/delete' "${op}" "${@}"
        ;;
    addfilter_truncate)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/truncate' '+' "${@}"
        ;;
    rmfilter_truncate)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/truncate' "${op}" "${@}"
        ;;
    addfilter_attr)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/attr' '+' "${@}"
        ;;
    rmfilter_attr)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/attr' "${op}" "${@}"
        ;;
    addfilter_chown)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/chown' '+' "${@}"
        ;;
    rmfilter_chown)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/chown' "${op}" "${@}"
        ;;
    addfilter_chgrp)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/chgrp' '+' "${@}"
        ;;
    rmfilter_chgrp)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/chgrp' "${op}" "${@}"
        ;;
    addfilter_ioctl)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/ioctl' '+' "${@}"
        ;;
    rmfilter_ioctl)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/ioctl' "${op}" "${@}"
        ;;
    addfilter_node)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/node' '+' "${@}"
        ;;
    rmfilter_node)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/node' "${op}" "${@}"
        ;;
    addfilter_tmpfile)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/tmpfile' '+' "${@}"
        ;;
    rmfilter_tmpfile)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if  [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/tmpfile' "${op}" "${@}"
        ;;
    addfilter_exec)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/exec' '+' "${@}"
        ;;
    rmfilter_exec)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_path 'filter/exec' "${op}" "${@}"
        ;;
    addfilter_net)
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_net 'filter/net' '+' "${@}"
        ;;
    rmfilter_net)
        local op='-'
        [ "${1}" == '--all' ] && op='^' && shift
        if [ ${#} -lt 1 ]; then
            echo >&2 "esyd: ${cmd} takes at least one extra argument"
            return 1
        fi
        _esyd_net 'filter/net' "${op}" "${@}"
        ;;
    help|'')
        if [ -t 1 ]; then
            esyd help | ${PAGER:-less}
            return $?
        fi

        cat <<EOF
esyd -- multi functional syd helper
Copyright (c) 2023, 2024 Ali Polatel <alip@chesswob.org>
SPDX-License-Identifier: GPL-3.0

# Subcommands
api
    Print syd API version number
check
    Return true if running under syd
lock
    Lock syd, esyd commands will no longer work
exec_lock
    Lock syd for all processes but the syd exec child
unlock
    Unlock the syd, make it available to all processes rather than just the syd exec child
info jq-args...
    Print syd sandbox state as JSON on standard output
    If "jq" is in PATH, pass the arguments and pipe the output to jq
load fd
    This command causes syd to read configuration from the given file descriptor
panic
    This command causes syd to exit immediately with code 127
reset
    This command causes syd to reset sandboxing to the default state
    Allowlists, denylists and filters are going to be cleared
stat
    Print syd sandbox state on standard error
exec cmd args..
    Execute a command outside the sandbox without sandboxing
kill glob
    Kill any attempt to execute a path matching the given glob pattern
force path hash action
    Force pre-execution verification of the given path using the given checksum
    Action must be exactly one of kill, stop or warn.
enabled, enabled_path
    Return true if path sandboxing is enabled
enable, enable_path
    Enable path sandboxing
disable, disable_path
    Disable path sandboxing
enabled_pid
    Return true if PID sandboxing is enabled
enable_pid
    Enable PID sandboxing
disable_pid
    Disable PID sandboxing
enabled_read
    Return true if read sandboxing is enabled
enable_read
    Enable read sandboxing
disable_read
    Disable read sandboxing
enabled_stat
    Return true if stat sandboxing is enabled
enable_stat
    Enable stat sandboxing
disable_stat
    Disable stat sandboxing
enabled_write
    Return true if write sandboxing is enabled
enable_write
    Enable write sandboxing
disable_write
    Disable write sandboxing
enabled_create
    Return true if create sandboxing is enabled
enable_create
    Enable create sandboxing
disable_create
    Disable create sandboxing
enabled_delete
    Return true if delete sandboxing is enabled
enable_delete
    Enable delete sandboxing
disable_delete
    Disable delete sandboxing
enabled_truncate
    Return true if truncate sandboxing is enabled
enable_truncate
    Enable truncate sandboxing
disable_truncate
    Disable truncate sandboxing
enabled_attr
    Return true if attr sandboxing is enabled
enable_attr
    Enable attr sandboxing
disable_attr
    Disable attr sandboxing
enabled_chown
    Return true if chown sandboxing is enabled
enable_chown
    Enable chown sandboxing
disable_chown
    Disable chown sandboxing
enabled_chgrp
    Return true if chgrp sandboxing is enabled
enable_chgrp
    Enable chgrp sandboxing
disable_chgrp
    Disable chgrp sandboxing
enabled_ioctl
    Return true if ioctl sandboxing is enabled
enable_ioctl
    Enable ioctl sandboxing
disable_ioctl
    Disable ioctl sandboxing
enabled_node
    Return true if node sandboxing is enabled
enable_node
    Enable node sandboxing
disable_node
    Disable node sandboxing
enabled_tmpfile
    Return true if tmpfile sandboxing is enabled
enable_tmpfile
    Enable tmpfile sandboxing
disable_tmpfile
    Disable tmpfile sandboxing
enabled_exec
    Return true if exec sandboxing is enabled
enable_exec
    Enable exec sandboxing
disable_exec
    Disable exec sandboxing
enabled_force
    Return true if force sandboxing is enabled
enable_force
    Enable force sandboxing
disable_force
    Disable force sandboxing
enabled_net
    Return true if network sandboxing is enabled
enable_net
    Enable network sandboxing
disable_net
    Disable network sandboxing
allow, allow_path glob
    Allow the given glob pattern for path sandboxing
disallow, disallow_path [--all] glob
    Removes the given glob pattern from the allowlist for path sandboxing
deny, deny_path glob
    Deny the given glob pattern for path sandboxing
nodeny, nodeny_path [--all] glob
    Removes the given glob pattern from the denylist for path sandboxing
allow_read glob
    Allow the given glob pattern for read sandboxing
disallow_read [--all] glob
    Removes the given glob pattern from the allowlist for read sandboxing
deny_read glob
    Deny the given glob pattern for read sandboxing
nodeny_read [--all] glob
    Removes the given glob pattern from the denylist for read sandboxing
allow_stat glob
    Allow the given glob pattern for stat sandboxing
disallow_stat [--all] glob
    Removes the given glob pattern from the allowlist for stat sandboxing
deny_stat glob
    Deny the given glob pattern for stat sandboxing
nodeny_stat [--all] glob
    Removes the given glob pattern from the denylist for stat sandboxing
allow_write glob
    Allow the given glob pattern for write sandboxing
disallow_write [--all] glob
    Removes the given glob pattern from the allowlist for write sandboxing
deny_write glob
    Deny the given glob pattern for write sandboxing
nodeny_write [--all] glob
    Removes the given glob pattern from the denylist for write sandboxing
allow_create glob
    Allow the given glob pattern for create sandboxing
disallow_create [--all] glob
    Removes the given glob pattern from the allowlist for create sandboxing
deny_create glob
    Deny the given glob pattern for create sandboxing
nodeny_create [--all] glob
    Removes the given glob pattern from the denylist for create sandboxing
allow_delete glob
    Allow the given glob pattern for delete sandboxing
disallow_delete [--all] glob
    Removes the given glob pattern from the allowlist for delete sandboxing
deny_delete glob
    Deny the given glob pattern for delete sandboxing
nodeny_delete [--all] glob
    Removes the given glob pattern from the denylist for delete sandboxing
allow_truncate glob
    Allow the given glob pattern for truncate sandboxing
disallow_truncate [--all] glob
    Removes the given glob pattern from the allowlist for truncate sandboxing
deny_truncate glob
    Deny the given glob pattern for truncate sandboxing
nodeny_truncate [--all] glob
    Removes the given glob pattern from the denylist for truncate sandboxing
allow_attr glob
    Allow the given glob pattern for attr sandboxing
disallow_attr [--all] glob
    Removes the given glob pattern from the allowlist for attr sandboxing
deny_attr glob
    Deny the given glob pattern for attr sandboxing
nodeny_attr [--all] glob
    Removes the given glob pattern from the denylist for attr sandboxing
allow_chown glob
    Allow the given glob pattern for chown sandboxing
disallow_chown [--all] glob
    Removes the given glob pattern from the allowlist for chown sandboxing
deny_chown glob
    Deny the given glob pattern for chown sandboxing
nodeny_chown [--all] glob
    Removes the given glob pattern from the denylist for chown sandboxing
allow_chgrp glob
    Allow the given glob pattern for chgrp sandboxing
disallow_chgrp [--all] glob
    Removes the given glob pattern from the allowlist for chgrp sandboxing
deny_chgrp glob
    Deny the given glob pattern for chgrp sandboxing
nodeny_chgrp [--all] glob
    Removes the given glob pattern from the denylist for chgrp sandboxing
allow_ioctl glob
    Allow the given glob pattern for ioctl sandboxing
disallow_ioctl [--all] glob
    Removes the given glob pattern from the allowlist for ioctl sandboxing
deny_ioctl glob
    Deny the given glob pattern for ioctl sandboxing
nodeny_ioctl [--all] glob
    Removes the given glob pattern from the denylist for ioctl sandboxing
allow_node glob
    Allow the given glob pattern for node sandboxing
disallow_node [--all] glob
    Removes the given glob pattern from the allowlist for node sandboxing
deny_node glob
    Deny the given glob pattern for node sandboxing
nodeny_node [--all] glob
    Removes the given glob pattern from the denylist for node sandboxing
allow_tmpfile glob
    Allow the given glob pattern for tmpfile sandboxing
disallow_tmpfile [--all] glob
    Removes the given glob pattern from the allowlist for tmpfile sandboxing
deny_tmpfile glob
    Deny the given glob pattern for tmpfile sandboxing
nodeny_tmpfile [--all] glob
    Removes the given glob pattern from the denylist for tmpfile sandboxing
allow_exec glob
    Allow the given glob pattern for exec sandboxing
disallow_exec [--all] glob
    Removes the given glob pattern from the allowlist for exec sandboxing
deny_exec glob
    Deny the given glob pattern for exec sandboxing
nodeny_exec [--all] glob
    Removes the given glob pattern from the denylist for exec sandboxing
allow_net [--connect|--send] glob|cidr!port[-port]
    Allow the given network address for network bind or connect sandboxing
disallow_net [--all] [--connect|--send] glob|cidr!port[-port]
    Removes the given network address (Ipv4,6), or the glob pattern (UNIX domain sockets)
    from the allowlist for network bind or connect sandboxing
deny_net [--connect|--send] glob|cidr!port[-port]
    Deny the given network address (Ipv4,6) or the glob pattern (UNIX domain sockets)
    for network bind or connect sandboxing
nodeny_net [--all] [--connect|--send] glob|cidr!port[-port]
    Removes the given network address (Ipv4,6) or the glob pattern (UNIX domain sockets)
    from the denylist for network bind or connect sandboxing
addfilter, addfilter_path glob
    Adds the given glob pattern to the list of access violation filters for path sandboxing
rmfilter, rmfilter_path [--all] glob
    Removes the given glob pattern from the list of access violation filters for path sandboxing
addfilter_read glob
    Adds the given glob pattern to the list of access violation filters for read sandboxing
rmfilter_read [--all] glob
    Removes the given glob pattern from the list of access violation filters for read sandboxing
addfilter_stat glob
    Adds the given glob pattern to the list of access violation filters for stat sandboxing
rmfilter_stat [--all] glob
    Removes the given glob pattern from the list of access violation filters for stat sandboxing
addfilter_write glob
    Adds the given glob pattern to the list of access violation filters for write sandboxing
rmfilter_write [--all] glob
    Removes the given glob pattern from the list of access violation filters for write sandboxing
addfilter_create glob
    Adds the given glob pattern to the list of access violation filters for create sandboxing
rmfilter_create [--all] glob
    Removes the given glob pattern from the list of access violation filters for create sandboxing
addfilter_delete glob
    Adds the given glob pattern to the list of access violation filters for delete sandboxing
rmfilter_delete [--all] glob
    Removes the given glob pattern from the list of access violation filters for delete sandboxing
addfilter_truncate glob
    Adds the given glob pattern to the list of access violation filters for truncate sandboxing
rmfilter_truncate [--all] glob
    Removes the given glob pattern from the list of access violation filters for truncate sandboxing
addfilter_attr glob
    Adds the given glob pattern to the list of access violation filters for attr sandboxing
rmfilter_attr [--all] glob
    Removes the given glob pattern from the list of access violation filters for attr sandboxing
addfilter_chown glob
    Adds the given glob pattern to the list of access violation filters for chown sandboxing
rmfilter_chown [--all] glob
    Removes the given glob pattern from the list of access violation filters for chown sandboxing
addfilter_chgrp glob
    Adds the given glob pattern to the list of access violation filters for chgrp sandboxing
rmfilter_chgrp [--all] glob
    Removes the given glob pattern from the list of access violation filters for chgrp sandboxing
addfilter_ioctl glob
    Adds the given glob pattern to the list of access violation filters for ioctl sandboxing
rmfilter_ioctl [--all] glob
    Removes the given glob pattern from the list of access violation filters for ioctl sandboxing
addfilter_node glob
    Adds the given glob pattern to the list of access violation filters for node sandboxing
rmfilter_node [--all] glob
    Removes the given glob pattern from the list of access violation filters for node sandboxing
addfilter_tmpfile glob
    Adds the given glob pattern to the list of access violation filters for tmpfile sandboxing
rmfilter_tmpfile [--all] glob
    Removes the given glob pattern from the list of access violation filters for tmpfile sandboxing
addfilter_exec glob
    Adds the given glob pattern to the list of access violation filters for exec sandboxing
rmfilter_exec [--all] glob
    Removes the given glob pattern from the list of access violation filters for exec sandboxing
addfilter_net glob|cidr!port[-port]
    Adds the network address (Ipv4,6) or the glob pattern (UNIX domain sockets)
    to the list of access violation filters for network sandboxing
rmfilter_net [--all] glob|cidr!port[-port]
    Removes the network address (Ipv4,6) or the glob pattern (UNIX domain sockets)
    from the list of access violation filters for network sandboxing
mem_max
    Set syd maximum per-process memory usage limit for memory sandboxing
    parse-size crate is used to parse the value so formatted strings are OK
vm_max
    Set syd maximum per-process virtual memory usage limit for memory sandboxing
    parse-size crate is used to parse the value so formatted strings are OK
kill_mem
    Send SIGKILL to process on Memory access violation
nokill_mem
    Do not send SIGKILL to process on Memory access violation
    Deny system call with ENOMEM
filter_mem
    Do not report access violations for memory sandboxing
unfilter_mem
    Report access violations for memory sandboxing
pid_max
    Set syd maximum process id limit for PID sandboxing
kill_pid
    Send SIGKILL to process on PID access violation
nokill_pid
    Do not send SIGKILL to process on PID access violation
    Deny system call with EACCES
filter_pid
    Do not report access violations for PID sandboxing
unfilter_pid
    Report access violations for PID sandboxing
EOF
        ;;
    *)
        echo >&2 "esyd: subcommand '${cmd}' unrecognised!"
        echo >&2 "Use 'esyd help' for a list of supported subcommands."
        return 1
        ;;
    esac
}

_esyd_path()
{
    local cmd="${1}"
    local op="${2}"

    case "${op}" in
    '+'|'-')
        ;;
    *)
        echo >&2 "esyd_path: invalid operation character '${op}'"
        return 1
        ;;
    esac

    shift 2

    local ret=0
    local path
    for path in "${@}"; do
        case "${path}" in
        /*)
            [ -c /dev/syd/"${cmd}${op}${path}" ] || ret=$?
            ;;
        *)
            echo >&2 "esyd_path: expects absolute path, got: ${path}"
            return 1
            ;;
        esac
    done
    return $ret
}

_esyd_net()
{
    local cmd="${1}"
    local op="${2}"

    case "${op}" in
    '+'|'-')
        ;;
    *)
        echo >&2 "esyd_net: invalid operation character '${op}'"
        return 1
        ;;
    esac

    shift 2

    local ret=0
    while [ ${#} -gt 0 ] ; do
        # syd does input validation so we don't do any here.
        [ -c "/dev/syd/${cmd}${op}${1}" ] || ret=$?
        shift
    done
    return $ret
}
