//
// Syd: rock-solid application kernel
// src/fs.rs: Filesystem utilities
//
// Copyright (c) 2023, 2024 Ali Polatel <alip@chesswob.org>
// Based in part upon uutils coreutils package's src/lib/features/fs.rs which is:
//   (c) Joseph Crail <jbcrail@gmail.com>
//   (c) Jian Zeng <anonymousknight96 AT gmail.com>
// Tests base based in part upon gnulib packages' tests/test-canonicalize.c which is:
//   (c) Free Software Foundation, Inc.
//
// SPDX-License-Identifier: GPL-3.0

//! Set of functions to manage files and symlinks

use std::{
    borrow::Cow,
    collections::VecDeque,
    ffi::{CStr, OsStr, OsString},
    fs::{metadata, set_permissions, File},
    hash::Hash,
    io::Write,
    ops::{Deref, RangeInclusive},
    os::{
        fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd, OwnedFd, RawFd},
        unix::{
            ffi::{OsStrExt, OsStringExt},
            fs::PermissionsExt,
        },
    },
    path::Path,
};

use bitflags::bitflags;
use btoi::btoi;
use nix::{
    errno::Errno,
    fcntl::{fcntl, openat2, readlinkat, FcntlArg, FdFlag, OFlag, OpenHow, ResolveFlag, SealFlag},
    libc::{
        mode_t, DT_BLK, DT_CHR, DT_DIR, DT_FIFO, DT_LNK, DT_REG, DT_SOCK, S_IFBLK, S_IFCHR,
        S_IFDIR, S_IFIFO, S_IFLNK, S_IFMT, S_IFREG, S_IFSOCK,
    },
    sched::CloneCb,
    sys::inotify::AddWatchFlags,
    unistd::Pid,
    NixPath,
};
use rustc_hash::{FxHashMap, FxHashSet};
use serde::{Serialize, Serializer};

use crate::{
    compat::{
        fstatx, getdents64, statx, FileStatx, STATX_INO, STATX_MNT_ID, STATX_MNT_ID_UNIQUE,
        STATX_MODE, STATX_SIZE, STATX_TYPE,
    },
    config::*,
    err::err2no,
    path::{XPath, XPathBuf, PATH_MAX},
    proc::{proc_fd, proc_tgid},
    sandbox::Flags,
};

/// MFD_CLOEXEC memfd_create(2) flag.
pub const MFD_CLOEXEC: nix::libc::c_uint = nix::libc::MFD_CLOEXEC;
/// MFD_ALLOW_SEALING memfd_create(2) flag.
pub const MFD_ALLOW_SEALING: nix::libc::c_uint = nix::libc::MFD_ALLOW_SEALING;
/// MFD_NOEXEC_SEAL memfd_create(2) flag.
pub const MFD_NOEXEC_SEAL: nix::libc::c_uint = nix::libc::MFD_NOEXEC_SEAL;
/// MFD_EXEC memfd_create(2) flag.
pub const MFD_EXEC: nix::libc::c_uint = nix::libc::MFD_EXEC;

/// Information to uniquely identify a file.
pub struct FileInformation {
    /// Inode number
    pub ino: u64,
    /// Device major number
    pub dev_major: u32,
    /// Device minor number
    pub dev_minor: u32,
    /// File mode
    pub mode: u16,
}

impl FileInformation {
    /// Get information for a given link.
    pub fn from_link(path: &XPath) -> Result<Self, Errno> {
        // Note, we deliberately don't pass STATX_MODE in flags here, as
        // the only caller canonicalize() do not need this field.
        let stat = retry_on_eintr(|| {
            statx(
                None,
                path,
                nix::libc::AT_SYMLINK_NOFOLLOW as nix::libc::c_uint,
                STATX_INO,
            )
        })?;

        Ok(Self {
            ino: stat.stx_ino,
            dev_major: stat.stx_dev_major,
            dev_minor: stat.stx_dev_minor,
            mode: 0,
        })
    }

    /// Get information from a given path.
    pub fn from_path(path: &XPath) -> Result<Self, Errno> {
        let stat = retry_on_eintr(|| statx(None, path, 0, STATX_MODE | STATX_INO))?;

        Ok(Self {
            ino: stat.stx_ino,
            dev_major: stat.stx_dev_major,
            dev_minor: stat.stx_dev_minor,
            mode: stat.stx_mode,
        })
    }
}

impl PartialEq for FileInformation {
    fn eq(&self, other: &Self) -> bool {
        self.ino == other.ino
            && self.dev_major == other.dev_major
            && self.dev_minor == other.dev_minor
    }
}

impl Eq for FileInformation {}

impl Hash for FileInformation {
    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
        self.ino.hash(state);
        self.dev_major.hash(state);
        self.dev_minor.hash(state);
    }
}

bitflags! {
    /// Path canonicalization options
    #[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
    pub struct FsFlags: u8 {
        /// All components of the path must exist.
        /// Without this option, the default is:
        /// Last component may exist, other components must exist.
        /// Conflicts with the option `MISS_LAST`.
        const MUST_PATH = 1 << 0;
        /// Last component must not exist, other components must exist.
        /// Without this option, the default is:
        /// Last component may exist, other components must exist.
        /// Conflicts with the option `MUST_PATH`.
        const MISS_LAST = 1 << 1;
        /// Do not follow symbolic links for the last path component.
        /// Symbolic links for parent components will be resolved.
        const NO_FOLLOW_LAST = 1 << 2;
        /// Do not permit the path resolution to succeed if any
        /// component of the resolution is not a descendant of the
        /// directory indicated by dirfd. This causes absolute symbolic
        /// links (and absolute values of pathname) to be rejected.
        /// Currently, this flag also disables magic-link resolution
        /// (see below). However, this may change in the future.
        /// Therefore, to ensure that magic links are not resolved, the
        /// caller should explicitly specify RESOLVE_NO_MAGICLINKS.
        const RESOLVE_BENEATH = 1 << 3;
        /// Do not resolve symbolic links for any of the path components.
        const NO_RESOLVE_PATH = 1 << 4;
        /// Do not resolve proc magic symbolic links.
        const NO_RESOLVE_PROC = 1 << 5;
        /// Do not traverse through mount points.
        const NO_RESOLVE_XDEV = 1 << 6;
    }
}

// Controls how missing components should be handled when canonicalizing a path.
#[derive(Clone, Copy, Debug, Eq, PartialEq)]
enum MissingHandling {
    // Last component may exist, other components must exist.
    // Default mode.
    Normal,

    // All components must exist.
    // Maps to FsFlags::MUST_PATH.
    Existing,

    // Last component must not exist, other componenets must exist.
    // Maps to FsFlags::MISS_LAST.
    Missing,
}

impl Default for MissingHandling {
    fn default() -> Self {
        Self::Normal
    }
}

impl From<FsFlags> for MissingHandling {
    fn from(flag: FsFlags) -> Self {
        if flag.contains(FsFlags::MUST_PATH) {
            Self::Existing
        } else if flag.contains(FsFlags::MISS_LAST) {
            Self::Missing
        } else {
            Self::Normal
        }
    }
}

impl FsFlags {
    /// Return `Errno` corresponding to magic link restrictions.
    pub fn magic_errno(&self) -> Errno {
        if self.intersects(Self::RESOLVE_BENEATH | Self::NO_RESOLVE_XDEV) {
            Errno::EXDEV
        } else if self.intersects(Self::NO_RESOLVE_PATH | Self::NO_RESOLVE_PROC) {
            Errno::ELOOP
        } else {
            Errno::EACCES
        }
    }

    /// Return true if symbolic links in last path component should be followed.
    pub fn follow_last(&self) -> bool {
        !self.contains(Self::NO_FOLLOW_LAST)
    }

    /// Return true if symbolic links in path should be resolved.
    pub fn resolve_path(&self) -> bool {
        !self.contains(Self::NO_RESOLVE_PATH)
    }

    /// Return true if magic /proc symbolic links should be resolved.
    pub fn resolve_proc(&self) -> bool {
        !self.intersects(Self::NO_RESOLVE_PROC | Self::NO_RESOLVE_XDEV | Self::RESOLVE_BENEATH)
    }

    /// Return true if the path must exist.
    pub fn must_exist(&self) -> bool {
        self.contains(Self::MUST_PATH)
    }

    /// Return true if the file may be missing.
    pub fn missing(&self) -> bool {
        self.contains(Self::MISS_LAST)
    }
}

impl Serialize for FsFlags {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        let mut flags: Vec<&str> = vec![];

        if self.is_empty() {
            return serializer.collect_seq(flags);
        }

        if self.contains(Self::MUST_PATH) {
            flags.push("must-path");
        }
        if self.contains(Self::MISS_LAST) {
            flags.push("miss-last");
        }
        if self.contains(Self::NO_FOLLOW_LAST) {
            flags.push("no-follow");
        }
        if self.contains(Self::RESOLVE_BENEATH) {
            flags.push("resolve-beneath");
        }
        if self.contains(Self::NO_RESOLVE_PATH) {
            flags.push("no-resolve-path");
        }
        if self.contains(Self::NO_RESOLVE_PROC) {
            flags.push("no-resolve-proc");
        }

        flags.sort();
        serializer.collect_seq(flags)
    }
}

/// This function creates a flock(2) lock.
pub fn flock_fd<F: AsRawFd>(fd: &F, exclusive: bool, wait: bool) -> Result<(), Errno> {
    let fd = fd.as_raw_fd();
    let mut op = if exclusive {
        nix::libc::LOCK_EX
    } else {
        nix::libc::LOCK_SH
    };
    if !wait {
        op |= nix::libc::LOCK_NB;
    }
    // SAFETY: nix deprecated direct flock,
    // and we cannot use the type-safe interface here.
    retry_on_eintr(|| Errno::result(unsafe { nix::libc::flock(fd, op) })).map(drop)
}

/// This function unlocks a flock(2) lock.
pub fn funlock_fd<F: AsRawFd>(fd: &F) -> Result<(), Errno> {
    let fd = fd.as_raw_fd();
    // SAFETY: nix deprecated direct flock,
    // and we cannot use the type-safe interface here.
    retry_on_eintr(|| Errno::result(unsafe { nix::libc::flock(fd, nix::libc::LOCK_UN) })).map(drop)
}

/// This function creates an Open File Description (OFD) lock.
pub fn lock_fd<F: AsRawFd>(fd: &F, write: bool, wait: bool) -> Result<(), Errno> {
    let fd = fd.as_raw_fd();
    #[allow(clippy::cast_possible_truncation)]
    let lock = nix::libc::flock {
        l_type: if write {
            nix::libc::F_WRLCK
        } else {
            nix::libc::F_RDLCK
        } as i16,
        l_whence: nix::libc::SEEK_SET as i16,
        l_start: 0,
        l_len: 1, // Lock the first byte.
        l_pid: 0, // The kernel sets this value.
    };
    retry_on_eintr(|| {
        fcntl(
            fd,
            if wait {
                FcntlArg::F_OFD_SETLKW(&lock)
            } else {
                FcntlArg::F_OFD_SETLK(&lock)
            },
        )
    })
    .map(drop)
}

/// This function releases an Open File Description (OFD) lock.
pub fn unlock_fd<F: AsRawFd>(fd: &F) -> Result<(), Errno> {
    let fd = fd.as_raw_fd();
    #[allow(clippy::cast_possible_truncation)]
    let lock = nix::libc::flock {
        l_type: nix::libc::F_UNLCK as i16,
        l_whence: nix::libc::SEEK_SET as i16,
        l_start: 0,
        l_len: 1, // Lock the first byte.
        l_pid: 0, // The kernel sets this value.
    };
    retry_on_eintr(|| fcntl(fd, FcntlArg::F_OFD_SETLK(&lock))).map(drop)
}

/// Retries a closure on `EINTR` error.
///
/// This function will call the provided closure, and if the closure
/// returns an `EINTR` error, it will retry the operation until it
/// succeeds or fails with a different error.
#[inline]
pub(crate) fn retry_on_eintr<F, T>(mut f: F) -> Result<T, Errno>
where
    F: FnMut() -> Result<T, Errno>,
{
    loop {
        match f() {
            Err(Errno::EINTR) => continue,
            result => return result,
        }
    }
}

/// Creates an anonymous, non-executable file that lives in memory, and
/// return an owned file-descriptor to it.
pub fn create_memfd(name: &[u8], flags: nix::libc::c_uint) -> Result<OwnedFd, Errno> {
    create_memfd_raw(name, flags).map(|fd| {
        // SAFETY: memfd_create returns a valid FD.
        unsafe { OwnedFd::from_raw_fd(fd) }
    })
}

/// Creates an anonymous, non-executable file that lives in memory, and
/// return a raw file-descriptor to it.
pub fn create_memfd_raw(name: &[u8], flags: nix::libc::c_uint) -> Result<RawFd, Errno> {
    // Ensure the name is null-terminated
    let c_name = if name.last() == Some(&0) {
        Cow::Borrowed(name)
    } else {
        let mut c_name = Vec::with_capacity(name.len() + 1);
        c_name.extend_from_slice(name);
        c_name.push(0);
        Cow::Owned(c_name)
    };

    // SAFETY: nix' MemFdCreateFlags don't define MFD_EXEC and MFD_NOEXEC_SEAL yet.
    let fd = unsafe { nix::libc::memfd_create(c_name.as_ptr().cast(), flags | MFD_CLOEXEC) };
    match Errno::result(fd) {
        Ok(fd) => Ok(fd),
        Err(Errno::EINVAL) => {
            // 1. Flags included unknown bits.
            // 2. Name was too long.
            // 3. Both MFD_HUGETLB and MFD_ALLOW_SEALING were
            //    specified in flags.
            // In our case only the first one is relevant and
            // indicates MFD_NOEXEC_SEAL is unsupported. To avoid
            // potential confusion, we return EOPNOTSUPP rather
            // EINVAL than in this case.
            Err(Errno::EOPNOTSUPP)
        }
        Err(errno) => Err(errno),
    }
}

/// Seals the memfd for writing.
pub fn seal_memfd<F: AsRawFd>(fd: &F) -> Result<(), Errno> {
    // Seal memory fd.
    fcntl(
        fd.as_raw_fd(),
        FcntlArg::F_ADD_SEALS(
            SealFlag::F_SEAL_SEAL
                | SealFlag::F_SEAL_WRITE
                | SealFlag::F_SEAL_SHRINK
                | SealFlag::F_SEAL_GROW,
        ),
    )
    .map(drop)
}

/// Sets or clears the append (O_APPEND) flag on a file descriptor.
pub fn set_append<F: AsFd>(fd: &F, state: bool) -> Result<(), Errno> {
    let fd = fd.as_fd().as_raw_fd();
    let flags = fcntl(fd, FcntlArg::F_GETFL)?;

    let mut new_flags = flags;
    if state {
        new_flags |= OFlag::O_APPEND.bits();
    } else {
        new_flags &= !OFlag::O_APPEND.bits();
    }

    fcntl(fd, FcntlArg::F_SETFL(OFlag::from_bits_truncate(new_flags))).map(drop)
}

/// Sets or clears the non-blocking (O_NONBLOCK) flag on a file descriptor.
pub fn set_nonblock<F: AsFd>(fd: &F, state: bool) -> Result<(), Errno> {
    let fd = fd.as_fd().as_raw_fd();
    let flags = fcntl(fd, FcntlArg::F_GETFL)?;

    let mut new_flags = flags;
    if state {
        new_flags |= OFlag::O_NONBLOCK.bits();
    } else {
        new_flags &= !OFlag::O_NONBLOCK.bits();
    }

    fcntl(fd, FcntlArg::F_SETFL(OFlag::from_bits_truncate(new_flags))).map(drop)
}

/// Sets or clears the close-on-exec (FD_CLOEXEC) flag on a file descriptor.
pub fn set_cloexec<Fd: AsRawFd>(fd: &Fd, state: bool) -> Result<(), Errno> {
    let fd = fd.as_raw_fd();
    let flags = fcntl(fd, FcntlArg::F_GETFD)?;

    let mut new_flags = flags;
    if state {
        new_flags |= FdFlag::FD_CLOEXEC.bits();
    } else {
        new_flags &= !FdFlag::FD_CLOEXEC.bits();
    }

    fcntl(fd, FcntlArg::F_SETFD(FdFlag::from_bits_truncate(new_flags))).map(drop)
}

/// Set pipe max size of the given pipe.
pub fn set_pipemax<Fd: AsRawFd>(fd: &Fd, size: nix::libc::c_int) -> Result<usize, Errno> {
    #[allow(clippy::cast_sign_loss)]
    fcntl(fd.as_raw_fd(), FcntlArg::F_SETPIPE_SZ(size)).map(|r| r as usize)
}

/// Check two fds point to the same open file description for the given processes.
pub fn is_same_fd(pid1: Pid, pid2: Pid, fd1: RawFd, fd2: RawFd) -> Result<bool, Errno> {
    const KCMP_FILE: u64 = 0;
    // SAFETY: There's no libc wrapper for kcmp.
    Ok(Errno::result(unsafe {
        nix::libc::syscall(
            nix::libc::SYS_kcmp,
            pid1.as_raw(),
            pid2.as_raw(),
            KCMP_FILE,
            fd1,
            fd2,
        )
    })? == 0)
}

/// Initializes a new fanotify group.
///
/// This function wraps the `fanotify_init` system call.
///
/// # Arguments
///
/// * `flags` - The flags to control the behavior of the fanotify group.
/// * `event_f_flags` - The file descriptor flags for the event.
///
/// # Returns
///
/// A `Result` containing the file descriptor of the new fanotify group
/// if the operation was successful, or an `Err` containing the `Errno`
/// if it failed.
pub fn fanotify_init(
    flags: nix::libc::c_uint,
    event_f_flags: nix::libc::c_uint,
) -> Result<OwnedFd, Errno> {
    // SAFETY: The `fanotify_init` function is inherently unsafe because
    // it performs a system call that initializes a new fanotify group.
    // We ensure safety by:
    // - Passing valid `flags` and `event_f_flags` arguments as expected
    // by the system call.
    // - Checking the return value of `fanotify_init` for errors and
    // converting it to a safe `Result` type.
    let fd = unsafe { nix::libc::fanotify_init(flags, event_f_flags) };

    if fd >= 0 {
        // SAFETY: fanotify_init returns a valid FD.
        Ok(unsafe { OwnedFd::from_raw_fd(fd) })
    } else {
        Err(Errno::last())
    }
}

/// Marks a file or directory for fanotify.
///
/// This function wraps the `fanotify_mark` system call, always passing
/// a null pointer for the path.
///
/// # Arguments
///
/// * `notify_fd` - The fanotify file descriptor.
/// * `flags`     - The flags to control the behavior of the mark.
/// * `mask`      - The event mask to specify which events to notify on.
/// * `dirfd`     - The file descriptor of the file being marked.
///
/// # Returns
///
/// A `Result` which is `Ok` if the operation was successful, or an
/// `Err` containing the `Errno` if it failed.
pub fn fanotify_mark<F: AsRawFd, P: ?Sized + NixPath>(
    notify_fd: &F,
    flags: nix::libc::c_uint,
    mask: u64,
    dirfd: Option<RawFd>,
    path: Option<&P>,
) -> Result<(), Errno> {
    fn with_opt_nix_path<P, T, F>(p: Option<&P>, f: F) -> Result<T, Errno>
    where
        P: ?Sized + NixPath,
        F: FnOnce(*const nix::libc::c_char) -> T,
    {
        match p {
            Some(path) => path.with_nix_path(|p_str| f(p_str.as_ptr())),
            None => Ok(f(std::ptr::null())),
        }
    }

    // SAFETY: The `fanotify_mark` function is inherently unsafe
    // because it involves raw pointers and system-level operations.
    // We ensure safety by:
    // - Handling the optional `path` argument using the
    // `with_opt_nix_path` function.
    // - Using `dirfd.unwrap_or(libc::AT_FDCWD)` to provide a
    // default value for `dirfd` if it's `None`.
    // - Checking the return value of `fanotify_mark` for errors and
    // converting it to a safe `Result` type.
    let res = with_opt_nix_path(path, |p| unsafe {
        nix::libc::fanotify_mark(
            notify_fd.as_raw_fd(),
            flags,
            mask,
            dirfd.unwrap_or(nix::libc::AT_FDCWD),
            p,
        )
    })?;

    Errno::result(res).map(drop)
}

/// Wrapper for inotify_add_watch.
pub fn inotify_add_watch<F: AsRawFd, P: ?Sized + NixPath>(
    fd: &F,
    path: &P,
    mask: AddWatchFlags,
) -> Result<nix::libc::c_int, Errno> {
    // SAFETY: We need this because nix' `WatchDescriptor` is opaque...
    let res = path.with_nix_path(|cstr| unsafe {
        nix::libc::inotify_add_watch(fd.as_raw_fd(), cstr.as_ptr(), mask.bits())
    })?;

    Errno::result(res).map(|wd| wd as nix::libc::c_int)
}

/// Returns file mode for the given file descriptor.
/// The file mode includes the file type.
pub fn fd_mode<F: AsFd>(fd: &F) -> Result<nix::libc::mode_t, Errno> {
    retry_on_eintr(|| fstatx(fd, STATX_MODE | STATX_TYPE)).map(|statx| statx.stx_mode.into())
}

/// Represents UNIX file types
#[derive(Copy, Clone, Debug, Eq, PartialEq)]
pub enum FileType {
    /// Reprensents a void file
    Nil,
    /// Represents regular file
    Reg,
    /// Represents directory
    Dir,
    /// Represents symbolic link
    Lnk,
    /// Represents FIFO
    Fifo,
    /// Represents socket
    Sock,
    /// Represents character device
    Chr,
    /// Represents block device
    Blk,
    /// Represents unknown file
    Unk,
}

impl FileType {
    /// Test whether file type represents a regular file.
    pub fn is_file(&self) -> bool {
        *self == Self::Reg
    }

    /// Test whether file type represents a directory.
    pub fn is_dir(&self) -> bool {
        *self == Self::Dir
    }

    /// Test whether file type represents a symlink.
    pub fn is_symlink(&self) -> bool {
        *self == Self::Lnk
    }

    /// Test whether file type represents a block device.
    pub fn is_block_device(&self) -> bool {
        *self == Self::Blk
    }

    /// Test whether file type represents a character device.
    pub fn is_char_device(&self) -> bool {
        *self == Self::Chr
    }

    /// Test whether file type represents a FIFO.
    pub fn is_fifo(&self) -> bool {
        *self == Self::Fifo
    }

    /// Test whether file type represents a socket.
    pub fn is_socket(&self) -> bool {
        *self == Self::Sock
    }

    /// Test whether file type represents an unknown file.
    pub fn is_unknown(&self) -> bool {
        *self == Self::Unk
    }
}

impl From<u8> for FileType {
    fn from(dt_type: u8) -> Self {
        match dt_type {
            DT_DIR => Self::Dir,
            DT_REG => Self::Reg,
            DT_LNK => Self::Lnk,
            DT_CHR => Self::Chr,
            DT_BLK => Self::Blk,
            DT_FIFO => Self::Fifo,
            DT_SOCK => Self::Sock,
            _ => Self::Unk,
        }
    }
}

impl From<mode_t> for FileType {
    fn from(mode: mode_t) -> Self {
        match mode & S_IFMT {
            S_IFBLK => Self::Blk,
            S_IFCHR => Self::Chr,
            S_IFDIR => Self::Dir,
            S_IFIFO => Self::Fifo,
            S_IFLNK => Self::Lnk,
            S_IFREG => Self::Reg,
            S_IFSOCK => Self::Sock,
            _ => Self::Unk,
        }
    }
}

/// Return the file type of the given file descriptor.
#[allow(clippy::cast_possible_truncation)]
pub fn fd_type<F: AsFd>(fd: &F) -> Result<FileType, Errno> {
    retry_on_eintr(|| fstatx(fd, STATX_TYPE))
        .map(|statx| FileType::from(mode_t::from(statx.stx_mode)))
}

/// Checks if the given file mode represents a sidechannel device.
///
/// A sidechannel device is defined as a character or block device that
/// is world-readable or world-writable. This function checks the file
/// mode to determine if it meets these criteria.
///
/// # Arguments
///
/// * `mode` - The mode of the file to check.
#[inline]
pub fn is_sidechannel_device(mode: nix::libc::mode_t) -> bool {
    mode & (nix::libc::S_IROTH | nix::libc::S_IWOTH) != 0
        && matches!(
            mode & nix::libc::S_IFMT,
            nix::libc::S_IFCHR | nix::libc::S_IFBLK
        )
}

/// Resolve a symbolic link honouring magic proc links.
#[inline]
pub fn resolve_symlink(
    pid: Pid,
    path: &XPath,
    options: FsFlags,
) -> Result<XPathBuf, (Errno, Option<FileType>)> {
    // SAFETY:
    // 1. Handle /proc/self and /proc/thread-self magic links.
    // 2. Handle RESOLVE_NO_SYMLINKS & RESOLVE_NO_MAGICLINKS.
    if path.is_root() {
        // Special case, `/` is never a symlink.
        Err((Errno::EINVAL, Some(FileType::Dir)))
    } else if path.is_proc_self(false) {
        Ok(XPathBuf::from_pid(pid))
    } else if path.is_proc_self(true) {
        let mut p = match proc_tgid(pid) {
            Ok(tgid) => XPathBuf::from_pid(tgid),
            Err(errno) => return Err((errno, Some(FileType::Lnk))),
        };
        p.push(b"task");
        p.push_pid(pid);
        Ok(p)
    } else {
        // All good, try to open with O_PATH+O_NOFOLLOW.
        // SAFETY: We do not resolve symlinks here.
        let fd = match safe_open_path::<BorrowedFd>(None, path, OFlag::O_NOFOLLOW) {
            Ok(fd) => fd,
            Err(errno) => return Err((errno, None)),
        };
        let file_type = match fd_type(&fd) {
            Ok(file_type) => file_type,
            Err(errno) => return Err((errno, None)),
        };
        if file_type != FileType::Lnk {
            // fd is not a symbolic link fd.
            return Err((Errno::EINVAL, Some(file_type)));
        }
        let link = match readlinkat(Some(fd.as_raw_fd()), "") {
            Ok(link) => XPathBuf::from(link),
            Err(errno) => return Err((errno, Some(FileType::Lnk))),
        };
        if !options.resolve_path() {
            // RESOLVE_NO_SYMLINKS -> ELOOP
            Err((Errno::ELOOP, Some(FileType::Lnk)))
        } else if link.is_absolute() && options.contains(FsFlags::RESOLVE_BENEATH) {
            // RESOLVE_BENEATH -> EXDEV
            Err((Errno::EXDEV, Some(FileType::Lnk)))
        } else {
            // RESOLVE_NO_MAGICLINKS is handled by proc_fd in canonicalize().
            Ok(link)
        }
    }
}

/// A safe version of clone that returns a PidFD,
/// and therefore is not subject to PID-recycling
/// races.
pub fn safe_clone(
    mut cb: CloneCb,
    stack: &mut [u8],
    flags: nix::libc::c_int,
    signal: Option<nix::libc::c_int>,
) -> Result<OwnedFd, Errno> {
    #[allow(clippy::cast_possible_truncation)]
    extern "C" fn callback(data: *mut CloneCb) -> nix::libc::c_int {
        // SAFETY: nix' version does not support CLONE_PIDFD.
        let cb: &mut CloneCb = unsafe { &mut *data };
        (*cb)() as nix::libc::c_int
    }

    let mut pid_fd: nix::libc::c_int = -1;
    let combined: nix::libc::c_int = flags | nix::libc::CLONE_PIDFD | signal.unwrap_or(0);
    // SAFETY: ditto.
    #[allow(clippy::missing_transmute_annotations)]
    let res = unsafe {
        let ptr = stack.as_mut_ptr().add(stack.len());
        let ptr_aligned = ptr.sub(ptr as usize % 16);
        nix::libc::clone(
            std::mem::transmute(callback as extern "C" fn(*mut Box<dyn FnMut() -> isize>) -> i32),
            ptr_aligned as *mut nix::libc::c_void,
            combined,
            std::ptr::addr_of_mut!(cb) as *mut nix::libc::c_void,
            &mut pid_fd,
        )
    };

    Errno::result(res).map(|_| {
        // SAFETY: clone with CLONE_PIDFD returns a valid FD.
        unsafe { OwnedFd::from_raw_fd(pid_fd) }
    })
}

/// Ensure the file we open is a regular file.
/// Ensure we do not block on a fifo if the file is one.
/// Returns (File, FileStatx) on success, Errno on failure.
pub fn safe_open_file<F: AsFd>(fd: Option<&F>, path: &XPath) -> Result<(File, FileStatx), Errno> {
    // Step 1: Open the file with O_PATH.
    let fd = safe_open_path(fd, path, OFlag::empty())?;

    // Step 2: Check the file type and bail if it's not a regular file.
    let statx = retry_on_eintr(|| fstatx(&fd, STATX_INO | STATX_TYPE | STATX_SIZE))?;
    let ftype = FileType::from(nix::libc::mode_t::from(statx.stx_mode));
    if !ftype.is_file() {
        return Err(Errno::ENOEXEC);
    }

    // Step 3: Reopen the file safely via /proc/self/fd.
    // Note, we cannot use RESOLVE_NO_{MAGIC,SYM}LINKS or O_NOFOLLOW here.
    let mut path = XPathBuf::from("/proc/self/fd");
    path.push_fd(fd.as_raw_fd());
    let how = OpenHow::new().flags(OFlag::O_RDONLY | OFlag::O_CLOEXEC | OFlag::O_NOCTTY);
    let fd = retry_on_eintr(|| {
        openat2(nix::libc::AT_FDCWD, &path, how).map(|fd|
                // SAFETY: openat2 returns a valid FD.
                unsafe { File::from_raw_fd(fd) })
    })?;

    // Step 4: Return the File and FileStatx.
    Ok((fd, statx))
}

/// Safely open and copy the given pathname into the file,
/// if the pathname exists. Returns the number of bytes copied.
pub fn safe_copy_if_exists<F: Write>(dst: &mut F, src: &XPath) -> Result<u64, Errno> {
    let mut src = if let Ok((src, _)) = safe_open_file::<BorrowedFd>(None, src) {
        src
    } else {
        return Ok(0);
    };
    #[allow(clippy::disallowed_methods)]
    std::io::copy(&mut src, dst).map_err(|e| err2no(&e))
}

// Return a safe OpenHow structure.
#[inline]
pub(crate) fn safe_open_how(flags: OFlag) -> OpenHow {
    // Note we leave the caller to handle O_NOCTTY,
    // because its use is invalid with O_PATH.
    OpenHow::new()
        .flags(flags | OFlag::O_CLOEXEC | OFlag::O_NOFOLLOW)
        .resolve(ResolveFlag::RESOLVE_NO_MAGICLINKS | ResolveFlag::RESOLVE_NO_SYMLINKS)
}

// Return the mount id and file type of a path safely.
#[allow(clippy::cast_possible_truncation)]
pub(crate) fn safe_stat_mount_id<F: AsFd>(
    fd: Option<&F>,
    path: &XPath,
    flags: OFlag,
) -> Result<(u64, FileType), Errno> {
    let mut mask = STATX_TYPE;
    mask |= if *HAVE_STATX_MNT_ID_UNIQUE {
        STATX_MNT_ID_UNIQUE
    } else {
        STATX_MNT_ID
    };
    let fd = safe_open_path(fd, path, flags)?;
    let statx = retry_on_eintr(|| fstatx(&fd, mask))?;
    Ok((
        statx.stx_mnt_id,
        FileType::from(mode_t::from(statx.stx_mode)),
    ))
}

// Open a path safely using O_PATH and return an OwnedFd.
pub(crate) fn safe_open_path<F: AsFd>(
    fd: Option<&F>,
    path: &XPath,
    flags: OFlag,
) -> Result<OwnedFd, Errno> {
    safe_open(fd, path, OFlag::O_PATH | flags)
}

// Open a path safely and return an OwnedFd.
pub(crate) fn safe_open<F: AsFd>(
    fd: Option<&F>,
    path: &XPath,
    flags: OFlag,
) -> Result<OwnedFd, Errno> {
    let fd = fd
        .as_ref()
        .map(|fd| fd.as_fd().as_raw_fd())
        .unwrap_or(nix::libc::AT_FDCWD);
    let how = safe_open_how(flags);
    retry_on_eintr(|| {
        openat2(fd, path, how).map(|fd|
                // SAFETY: openat2 returns a valid FD.
                unsafe { OwnedFd::from_raw_fd(fd) })
    })
}

/// Get an extended attribute value.
///
/// # Arguments
/// - `fd`    - The file descriptor of the file.
/// - `name`  - The name of the extended attribute.
/// - `value` - The buffer to store the attribute value,
///             if None do a check run.
///
/// # Returns
/// - `Result<usize>` - The size of the extended attribute value.
///
/// # Errors
/// Returns an error if the operation fails.
///
/// # Safety
/// This function is unsafe as it directly interfaces with the libc function `fgetxattr`.
///
/// # See Also
/// [`fgetxattr`](https://man7.org/linux/man-pages/man2/fgetxattr.2.html)
pub fn fgetxattr<F: AsRawFd, P: ?Sized + NixPath>(
    fd: &F,
    name: &P,
    value: Option<&mut [u8]>,
) -> Result<usize, Errno> {
    let (value, len) = match value {
        Some(v) => (
            v.as_mut_ptr() as *mut nix::libc::c_void,
            v.len() as nix::libc::size_t,
        ),
        None => (std::ptr::null_mut(), 0),
    };

    // SAFETY: nix lacks a wrapper for fgetxattr.
    let res = name.with_nix_path(|name_ptr| unsafe {
        nix::libc::fgetxattr(fd.as_raw_fd(), name_ptr.as_ptr(), value, len)
    })?;

    #[allow(clippy::cast_sign_loss)]
    if res == -1 {
        Err(Errno::last())
    } else {
        Ok(res as usize)
    }
}

/// Set an extended attribute value.
///
/// # Arguments
/// - `fd`    - The file descriptor of the file.
/// - `name`  - The name of the extended attribute.
/// - `value` - The buffer containing the attribute value.
/// - `flags` - Flags to control the operation.
///
/// # Returns
/// - `Result<()>` - Returns an Ok result if the operation succeeds.
///
/// # Errors
/// Returns an error if the operation fails.
///
/// # Safety
/// This function is unsafe as it directly interfaces with the libc function `fsetxattr`.
///
/// # See Also
/// [`fsetxattr`](https://man7.org/linux/man-pages/man2/fsetxattr.2.html)
pub fn fsetxattr<F: AsRawFd, P: ?Sized + NixPath>(
    fd: &F,
    name: &P,
    value: &[u8],
    flags: i32,
) -> Result<(), Errno> {
    // SAFETY: nix lacks a wrapper for fsetxattr.
    let res = name.with_nix_path(|name_ptr| unsafe {
        nix::libc::fsetxattr(
            fd.as_raw_fd(),
            name_ptr.as_ptr(),
            value.as_ptr() as *const nix::libc::c_void,
            value.len() as nix::libc::size_t,
            flags as nix::libc::c_int,
        )
    })?;

    Errno::result(res).map(drop)
}

/// Remove an extended attribute value.
///
/// # Arguments
/// - `fd`    - The file descriptor of the file.
/// - `name`  - The name of the extended attribute.
///
/// # Returns
/// - `Result<()>` - Returns an Ok result if the operation succeeds.
///
/// # Errors
/// Returns an error if the operation fails.
///
/// # Safety
/// This function is unsafe as it directly interfaces with the libc function `fremovexattr`.
///
/// # See Also
/// [`fremovexattr`](https://man7.org/linux/man-pages/man2/fremovexattr.2.html)
pub fn fremovexattr<F: AsRawFd, P: ?Sized + NixPath>(fd: &F, name: &P) -> Result<(), Errno> {
    // SAFETY: nix lacks a wrapper for fremovexattr.
    let res = name.with_nix_path(|name_ptr| unsafe {
        nix::libc::fremovexattr(fd.as_raw_fd(), name_ptr.as_ptr())
    })?;

    Errno::result(res).map(drop)
}

/// Deny access to user.syd* extended attributes.
/// # Safety
/// Dereferences name after a NULL check.
/// If name is not NULL, it must be a valid NUL-terminated C-String.
/// # Security
/// Denies with ENODATA for stealth.
pub unsafe fn denyxattr(name: *const nix::libc::c_char) -> Result<(), Errno> {
    const SYD_XATTR: &[u8] = b"user.syd.";

    if name.is_null() {
        Ok(())
    } else {
        // SAFETY: The pointer from CStr is guaranteed to be valid and
        // null-terminated.
        if unsafe {
            nix::libc::strncmp(
                name,
                SYD_XATTR.as_ptr() as *const nix::libc::c_char,
                SYD_XATTR.len(),
            )
        } == 0
        {
            Err(Errno::ENODATA)
        } else {
            Ok(())
        }
    }
}

/// Filters out extended attribute names that start with "user.syd".
///
/// # Arguments
///
/// * `buf` - A buffer containing the extended attribute names as
///           null-terminated strings.
/// * `n` - The length of valid data in the buffer.
pub fn filterxattr(buf: &[u8], n: usize) -> Result<Vec<u8>, Errno> {
    let mut filtered_buf = Vec::new();
    let mut start = 0;

    #[allow(clippy::arithmetic_side_effects)]
    while start < n {
        if let Some(end) = buf[start..].iter().position(|&c| c == 0) {
            let name = &buf[start..start + end + 1]; // +1 to include the null terminator
            let cstr = CStr::from_bytes_with_nul(name).or(Err(Errno::E2BIG))?;
            if !cstr.to_bytes().starts_with(b"user.syd.") {
                filtered_buf.extend_from_slice(name);
            }
            start += end + 1;
        } else {
            break;
        }
    }

    Ok(filtered_buf)
}

/// Get secure bytes using the OS random number generator.
pub fn getrandom(size: usize) -> Result<Vec<u8>, Errno> {
    let mut buf = vec![0u8; size];
    fillrandom(&mut buf)?;
    Ok(buf)
}

/// Fill the given buffer using the OS random number generator.
pub fn fillrandom(buf: &mut [u8]) -> Result<(), Errno> {
    let size = buf.len();
    let mut tread = 0;

    #[allow(clippy::arithmetic_side_effects)]
    #[allow(clippy::cast_sign_loss)]
    while tread < size {
        // SAFETY: nix has no wrapper for getrandom yet!
        match Errno::result(unsafe {
            nix::libc::getrandom(
                buf.as_mut_ptr().add(tread) as *mut nix::libc::c_void,
                size - tread,
                nix::libc::GRND_RANDOM,
            )
        }) {
            Ok(nread) => tread += nread as usize,
            Err(Errno::EINTR) => {} // Interrupted, try again.
            Err(errno) => return Err(errno),
        }
    }

    Ok(())
}

/// Return a random `u64` within the given inclusive range using the OS
/// random number generator.
#[allow(clippy::arithmetic_side_effects)]
pub fn randint(range: RangeInclusive<u64>) -> Result<u64, Errno> {
    let start = *range.start();
    let end = *range.end();

    if start == end {
        return Ok(start);
    }

    let range_size = end.wrapping_sub(start).wrapping_add(1);
    if range_size == 0 {
        // The range covers the entire u64 range, return a random u64.
        let mut buf = [0u8; 8];
        fillrandom(&mut buf)?;
        return Ok(u64::from_le_bytes(buf));
    }

    // Calculate the maximum value to avoid modulo bias.
    let max = u64::MAX - (u64::MAX % range_size);

    loop {
        let mut buf = [0u8; 8];
        fillrandom(&mut buf)?;
        let rnd = u64::from_le_bytes(buf);

        if rnd < max {
            return Ok(rnd % range_size + start);
        }
    }
}

/// Return a random unprivileged port number using the OS random number
/// generator.
#[allow(clippy::cast_possible_truncation)]
#[inline]
pub fn randport() -> Result<u16, Errno> {
    randint(1025..=0xFFFF).map(|port| port as u16)
}

/// Check if the given File is executable.
pub(crate) fn is_executable<F: AsFd>(file: &F) -> bool {
    // SAFETY: Our nix version does not have AtFlags::AT_EACCES (TODO).
    Errno::result(unsafe {
        nix::libc::faccessat(
            file.as_fd().as_raw_fd(),
            b"\0".as_ptr().cast(),
            nix::libc::X_OK,
            nix::libc::AT_EACCESS | nix::libc::AT_EMPTY_PATH,
        )
    })
    .map(drop)
    .is_ok()
}

/// Parse a FD from a Path.
#[inline]
pub(crate) fn parse_fd(path: &XPath) -> Result<RawFd, Errno> {
    btoi::<RawFd>(path.as_bytes()).or(Err(Errno::EBADF))
}

/// Read a symbolic link and return a `XPathBuf`.
// TODO: Move to compat.rs
pub fn readlink(path: &XPath) -> Result<XPathBuf, Errno> {
    // Initialize path on stack.
    let mut buf = [0u8; PATH_MAX];

    #[allow(clippy::cast_sign_loss)]
    let size = path.with_nix_path(|cstr|
        // SAFETY: nix' readlink is not efficient for our usecase.
        unsafe {
            Errno::result(nix::libc::readlink(
                cstr.as_ptr(),
                buf.as_mut_ptr().cast(),
                PATH_MAX,
            ))
    })?? as usize;

    // SAFETY: If return value equals buffer size, truncation may have
    // occured. Technically this can never happen because we pass a
    // buffer with size PATH_MAX but better safe than sorry...
    if size >= PATH_MAX {
        return Err(Errno::ENAMETOOLONG);
    }

    Ok((&buf[..size]).into())
}

/// Create file and write the given content.
pub fn cat<P: AsRef<Path>, T: AsRef<[u8]>>(path: P, content: T) -> std::io::Result<()> {
    let mut file = File::create(path)?;
    file.write_all(content.as_ref())?;
    Ok(())
}

/// Make a file executable.
pub fn chmod_x<P: AsRef<Path>>(path: P) -> std::io::Result<()> {
    // Set permissions to make path executable.
    let metadata = metadata(path.as_ref())?;
    let mut permissions = metadata.permissions();
    permissions.set_mode(0o700); // This sets the file executable for the owner (rwx).
    set_permissions(path.as_ref(), permissions)
}

#[derive(Debug, PartialEq)]
enum PathComponent {
    //We handle {Root,Cur}Dir transparently for efficiency.
    //RootDir,
    //CurDir,
    ParentDir,
    Normal(OsString),
}

/// Generates a VecDeque of path components, optimized for performance and correctness.
/// - Assumes handling of the initial RootDir is done by the caller.
/// - Directly walks the path bytes to construct components, efficiently skipping multiple leading slashes.
/// - Returns an additional bool to check for trailing slash.
#[inline]
fn path_components(path: &XPath) -> (VecDeque<PathComponent>, bool) {
    let path_bytes = path.as_os_str().as_bytes();
    let mut components = VecDeque::with_capacity(16);

    // Create a new searcher for the '/' character.
    let searcher = memchr::arch::all::memchr::One::new(b'/');

    let mut start = 0;
    #[allow(clippy::arithmetic_side_effects)]
    for pos in searcher.iter(path_bytes) {
        if pos > start {
            match &path_bytes[start..pos] {
                b"." => {} // Skip current directory component.
                b".." => components.push_back(PathComponent::ParentDir),
                component => {
                    components.push_back(PathComponent::Normal(OsStr::from_bytes(component).into()))
                }
            }
        }
        start = pos + 1; // Move past the '/'.
    }

    // Handle the last component if it's not ended by a slash.
    #[allow(clippy::arithmetic_side_effects)]
    let slash = if start < path_bytes.len() {
        match &path_bytes[start..] {
            b"." => {
                // Skip current directory component.
                !components.is_empty() && start >= 1 && path_bytes[start - 1] == b'/'
            }
            b".." => {
                components.push_back(PathComponent::ParentDir);
                false
            }
            component => {
                components.push_back(PathComponent::Normal(OsStr::from_bytes(component).into()));
                false
            }
        }
    } else {
        true
    };

    (components, slash)
}

/// Adds to a VecDeque of (owning) path components, optimized for performance and correctness.
/// - Assumes handling of the initial RootDir is done by the caller.
/// - Directly walks the path bytes to construct components, efficiently skipping multiple leading slashes.
#[inline]
fn path_components2(path: &XPath, components: &mut VecDeque<PathComponent>) {
    let path_bytes = path.as_os_str().as_bytes();

    // Create a new searcher for the '/' character.
    let searcher = memchr::arch::all::memchr::One::new(b'/');

    let mut last_pos = path_bytes.len();
    #[allow(clippy::arithmetic_side_effects)]
    for pos in searcher.iter(path_bytes).rev() {
        match &path_bytes[pos + 1..last_pos] {
            b"" | b"." => {} // Skip current directory and empty components.
            b".." => components.push_front(PathComponent::ParentDir),
            component => components.push_front(PathComponent::Normal(OsString::from_vec(
                component.to_vec(),
            ))),
        }
        last_pos = pos;
    }

    // Handle the remaining component before the first slash (or the
    // only component if no slashes).
    match &path_bytes[..last_pos] {
        b"" | b"." => {} // Skip current directory and empty components.
        b".." => components.push_front(PathComponent::ParentDir),
        component => components.push_front(PathComponent::Normal(OsString::from_vec(
            component.to_vec(),
        ))),
    }
}

/// Return the canonical, absolute form of a path
///
/// This function is a generalization of [`std::fs::canonicalize`] that
/// allows controlling how symbolic links are resolved and how to deal
/// with missing components. It returns the canonical, absolute form of
/// a path.
///
/// The `flag` parameter determines the sandbox restrictions to apply.
///
/// The `miss_mode` parameter controls how missing components are handled.
///
/// The `resolve` is a boolean parameter which controls whether
/// the last component should be resolved or not. Remaining components
/// are always resolved.
#[allow(clippy::cognitive_complexity)]
pub fn canonicalize(
    pid: Pid,
    path: &XPath,
    cwd: Option<&XPath>,
    options: FsFlags,
    flags: Flags,
) -> Result<(XPathBuf, Option<FileType>), Errno> {
    let resolve_beneath = options.contains(FsFlags::RESOLVE_BENEATH);
    let mut result = if path.is_relative() {
        if let Some(cwd) = cwd {
            if cwd.is_relative() {
                return Err(Errno::EINVAL);
            }
            XPathBuf::from(cwd)
        } else if path.is_empty() {
            return Err(Errno::ENOENT);
        } else {
            return Err(Errno::EINVAL);
        }
    } else if resolve_beneath {
        // RESOLVE_BENEATH rejects absolute values of pathname.
        return Err(Errno::EXDEV);
    } else if path.is_root() {
        // Special case, `/` is never a symlink.
        // This must be done after the RESOLVE_BENEATH check!
        return Ok((XPathBuf::from("/"), Some(FileType::Dir)));
    } else {
        XPathBuf::from("/")
    };

    // What do we do when a component is missing?
    let miss_mode = MissingHandling::from(options);
    // Do we want to resolve symbolic links for the last component?
    let no_follow_last = !options.follow_last();
    // Do we want to resolve magic /proc symbolic links?
    let resolve_proc = options.resolve_proc();
    // Do we want to traverse through mount points?
    let resolve_xdev = !options.contains(FsFlags::NO_RESOLVE_XDEV);

    // Used by proc_fd:
    // 1. Negated PID skips FD parsing.
    // 2. Deny regardless of PID if RESOLVE_NO_MAGICLINKS.
    // `magic_errno` determines between EXDEV and ELOOP.
    #[allow(clippy::arithmetic_side_effects)]
    let pid_check = Pid::from_raw(if resolve_proc { -pid.as_raw() } else { 0 });
    let pid_errno = options.magic_errno();

    // `..` restriction for path traversal.
    let deny_dotdot = flags.contains(Flags::FL_DENY_DOTDOT);
    // Restriction for /proc magic links.
    let restrict_magiclinks = match pid_errno {
        Errno::ELOOP => true,
        Errno::EXDEV => false,
        _ => !flags.contains(Flags::FL_ALLOW_UNSAFE_MAGICLINKS),
    };

    // Determine reference mount id for RESOLVE_NO_XDEV.
    let mut mount_data = if resolve_xdev {
        None
    } else if let Some(cwd) = cwd {
        let (mid, ft) = safe_stat_mount_id::<BorrowedFd>(None, cwd, OFlag::O_DIRECTORY)?;
        if ft == FileType::Lnk {
            // SAFETY: symlink appeared out-of-nowhere, deny!
            return Err(Errno::ELOOP);
        }
        let mut map = FxHashMap::default();
        map.insert(XPathBuf::from(cwd), (mid, ft));
        Some((mid, map))
    } else {
        // Jumping to "/" is ok, but later components cannot cross.
        return Err(Errno::EXDEV);
    };

    // Symbolic Lnk Loop Detection.
    const SYMLINKS_TO_LOOK_FOR_LOOPS: u8 = 16;
    let mut followed_symlinks = 0;
    let mut visited_files: Option<FxHashSet<FileInformation>> = None;

    let mut file_type = Some(FileType::Nil);
    let (mut parts, has_to_be_directory) = path_components(path);
    while let Some(part) = parts.pop_front() {
        // Check for RESOLVE_BENEATH and RESOLVE_NO_XDEV.
        // Temporary jumps are not allowed!
        if resolve_beneath {
            if let Some(cwd) = cwd {
                if !result.deref().descendant_of(cwd.as_bytes()) {
                    return Err(Errno::EXDEV);
                }
            } else {
                return Err(Errno::EXDEV);
            }
        }
        file_type = Some(FileType::Nil);
        if let Some((mount_id, ref mut mount_map)) = mount_data {
            // Jumping to "/" is ok, but later components cannot cross.
            if !result.is_root() {
                let id = if let Some((id, _)) = mount_map.get(&result).copied() {
                    id
                } else {
                    let flags = if !parts.is_empty() {
                        OFlag::O_DIRECTORY
                    } else {
                        OFlag::empty()
                    };
                    let (id, ft) = safe_stat_mount_id::<BorrowedFd>(None, result.deref(), flags)?;
                    file_type = Some(ft);
                    if ft == FileType::Lnk {
                        // SAFETY: symlink appeared out-of-nowhere, deny!
                        return Err(Errno::ELOOP);
                    }
                    mount_map.insert(result.clone(), (id, ft));
                    id
                };
                if id != mount_id {
                    return Err(Errno::EXDEV);
                }
            }
        }

        match part {
            PathComponent::Normal(p) => result.push(p.as_bytes()),
            PathComponent::ParentDir => {
                if deny_dotdot {
                    return Err(Errno::EACCES);
                }
                // SAFETY:
                // 1. `result` is a normalized absolute path.
                // 2. `result` does not have a trailing slash.
                unsafe { result.pop_unchecked() };
                if result.is_root() {
                    // RootDir as part of walking up path of
                    // an earlier symlink.
                    continue;
                }
            }
        }

        // We're resolving symbolic links for all path components but last.
        // The resolution of the last component depends on the NO_FOLLOW_LAST option.
        let last = parts.is_empty();
        if last && no_follow_last && !has_to_be_directory {
            break; // Nothing left to do, break out.
        }

        match resolve_symlink(pid, result.deref(), options) {
            Ok(link_path) => {
                let rel = link_path.is_relative();

                // SAFETY:
                // 1. NO_MAGICLINKS unless trace/allow_unsafe_magiclinks:1.
                //    Continue in case:
                //    (a) pipe:42 socket:42 etc, special paths.
                //    (b) file descriptors that point to deleted paths.
                // 2. Skip for EXDEV because jump through magic-link to
                //    same procfs must be possible for RESOLVE_BENEATH.
                match proc_fd(pid_check, result.deref(), restrict_magiclinks) {
                    Ok(Some(_)) if rel || link_path.ends_with(b" (deleted)") => {
                        if last {
                            // TODO: FileType::MagicLnk!
                            return Ok((result, Some(FileType::Lnk)));
                        }
                        continue;
                    }
                    Err(_) => return Err(pid_errno),
                    _ => {}
                };

                //// Symbolic Lnk Loop Detection.
                // SAFETY: SYMLINKS_TO_LOOK_FOR_LOOPS is much smaller than u8::MAX.
                #[allow(clippy::arithmetic_side_effects)]
                if followed_symlinks < SYMLINKS_TO_LOOK_FOR_LOOPS {
                    followed_symlinks += 1;
                } else {
                    let stat = FileInformation::from_link(result.deref())?;
                    if let Some(ref mut visited_files) = visited_files {
                        if !visited_files.insert(stat) {
                            return Err(Errno::ELOOP);
                        }
                    } else {
                        // Allocate the FxHashSet only when it's really necessary.
                        visited_files = Some(FxHashSet::from_iter([stat]));
                    }
                }
                ////

                path_components2(link_path.deref(), &mut parts);
                if rel {
                    // SAFETY:
                    // 1. `result` is a normalized absolute path.
                    // 2. `result` does not have a trailing slash.
                    unsafe { result.pop_unchecked() };
                } else {
                    // 1. RootDir as part of resolving a symlink to an absolute path!
                    // 2. `result` is an absolute path so truncating to 1 effectively
                    //    leaves behind `/` only.
                    result.truncate(1);
                }
            }
            Err((Errno::EINVAL, ft)) => {
                // File is not a symbolic link,
                // record potential file type,
                // and continue.
                file_type = ft;
            }
            Err((errno, ft)) => match miss_mode {
                MissingHandling::Existing => return Err(errno),
                MissingHandling::Normal if !parts.is_empty() => return Err(errno),
                _ => file_type = ft,
            },
        }
    }

    // Check for RESOLVE_BENEATH and RESOLVE_NO_XDEV.
    if resolve_beneath {
        if let Some(cwd) = cwd {
            if !result.deref().descendant_of(cwd.as_bytes()) {
                return Err(Errno::EXDEV);
            }
        } else {
            return Err(Errno::EXDEV);
        }
    }

    // Assert directory requirement with O_DIRECTORY,
    // unless the entry has to be missing.
    let flags = if has_to_be_directory {
        OFlag::O_PATH | OFlag::O_DIRECTORY
    } else {
        OFlag::O_PATH
    };

    if let Some((mount_id, mount_map)) = mount_data {
        let id = if let Some((id, ft)) = mount_map.get(&result).copied() {
            // SAFETY: file_type cannot be Lnk here!
            file_type = Some(ft);
            id
        } else {
            match (
                safe_stat_mount_id::<BorrowedFd>(None, result.deref(), flags),
                miss_mode,
            ) {
                (Ok((_, FileType::Lnk)), _) if !no_follow_last => {
                    // SAFETY: symlink appeared out-of-nowhere, deny!
                    return Err(Errno::ELOOP);
                }
                (Ok((id, ft)), _) => {
                    file_type = Some(ft);
                    id
                }
                (Err(Errno::ELOOP), _) => {
                    // SAFETY: symlink appeared out-of-nowhere, deny!
                    return Err(Errno::ELOOP);
                }
                (Err(Errno::ENOENT), MissingHandling::Existing) => {
                    return Err(Errno::ENOENT);
                }
                (Err(Errno::ENOTDIR), MissingHandling::Existing | MissingHandling::Normal)
                    if has_to_be_directory =>
                {
                    return Err(Errno::ENOTDIR);
                }
                _ => return Err(Errno::EXDEV),
            }
        };
        if id != mount_id {
            return Err(Errno::EXDEV);
        }
    }

    // Check file type without resolving symlinks.
    // Raise ENOTDIR or ENOENT as necessary.
    if file_type == Some(FileType::Nil) {
        file_type = match (
            safe_open_path::<BorrowedFd>(None, result.deref(), flags),
            miss_mode,
        ) {
            (Ok(fd), _) => {
                let file_type = fd_type(&fd).ok();
                if !no_follow_last && file_type == Some(FileType::Lnk) {
                    // SAFETY: symlink appeared out-of-nowhere, deny!
                    return Err(Errno::ELOOP);
                }
                file_type
            }
            (Err(Errno::ELOOP), _) => {
                // SAFETY: symlink appeared out-of-nowhere, deny!
                return Err(Errno::ELOOP);
            }
            (Err(Errno::ENOENT), MissingHandling::Existing) => {
                return Err(Errno::ENOENT);
            }
            (Err(Errno::ENOTDIR), MissingHandling::Existing | MissingHandling::Normal)
                if has_to_be_directory =>
            {
                return Err(Errno::ENOTDIR)
            }
            // SAFETY: No check for missing because EEXIST return
            // value may be misused to identify hidden paths.
            _ => None,
        };
    }

    // Keep trailing slash as necessary.
    if has_to_be_directory {
        if matches!(
            miss_mode,
            MissingHandling::Existing | MissingHandling::Normal
        ) && !matches!(file_type, None | Some(FileType::Dir))
        {
            return Err(Errno::ENOTDIR);
        }
        result.append_byte(b'/');
    }

    Ok((result, file_type))
}

/// Searches for a name within a directory.
///
/// `name` is matched literally and exactly against file names.
/// Directory entries are appended a trailing slash before matching.
/// Symlink entries are appended an `@` character before matching.
pub fn grep(dir: &XPath, name: &[u8]) -> Option<XPathBuf> {
    let dir = File::open(dir.as_path()).ok()?;
    let name = XPath::from_bytes(name);
    loop {
        let mut entries = match getdents64(&dir, 128) {
            Ok(entries) => entries,
            Err(_) => return None,
        };
        for entry in &mut entries {
            let mut path = XPathBuf::from(entry.name_bytes());
            // Append a trailing slash for directories.
            if entry.is_dir() {
                path.append_byte(b'/');
            } else if entry.is_symlink() {
                path.append_byte(b'@');
            } else if entry.is_block_device() {
                path.append_byte(b'!');
            } else if entry.is_char_device() {
                path.append_byte(b'$');
            } else if entry.is_fifo() {
                path.append_byte(b'|');
            } else if entry.is_socket() {
                path.append_byte(b'~');
            }
            if *path == *name || (name.len() == 1 && path.ends_with(name.as_bytes())) {
                return Some(path);
            }
        }
    }
}

#[cfg(test)]
mod tests {
    use std::{
        fs::{self, OpenOptions},
        os::{
            fd::AsRawFd,
            unix::{
                fs::{symlink, OpenOptionsExt},
                net::UnixStream,
            },
        },
        process::{exit, Command},
        thread::sleep,
        time::{Duration, SystemTime},
    };

    use nix::{
        fcntl::open,
        sys::{
            signal::{kill, Signal},
            stat::Mode,
        },
        unistd::{fork, pause, ForkResult},
    };

    use super::*;
    use crate::xpath;

    type TestResult = Result<(), Box<dyn std::error::Error>>;

    // A helper function to get the current atime of a file
    fn get_atime<P: AsRef<Path>>(path: P) -> SystemTime {
        let metadata = fs::metadata(path).expect("Failed to get metadata");
        metadata.accessed().expect("Failed to get accessed time")
    }

    // Helper function to assert that the atime of a file or directory has not changed
    fn assert_atime_unchanged<P: AsRef<Path>, F>(path: P, func: F)
    where
        F: FnOnce() -> Result<(XPathBuf, Option<FileType>), Errno>,
    {
        let original_atime_f = get_atime(&path);
        let original_atime_p = get_atime(path.as_ref().parent().unwrap());
        sleep(Duration::from_secs(7));
        assert!(
            func().is_ok(),
            "canonicalize {} failed",
            path.as_ref().display()
        );
        // We allow a 1-second tolerance since some filesystems do not have nanosecond precision.
        let new_atime_f = get_atime(&path);
        let new_atime_p = get_atime(path.as_ref().parent().unwrap());
        assert!(
            new_atime_f <= original_atime_f + Duration::new(1, 0),
            "The atime of the file should not have significantly changed."
        );
        assert!(
            new_atime_p <= original_atime_p + Duration::new(1, 0),
            "The atime of the parent dir should not have significantly changed."
        );
    }

    // std::fs::remove_dir_all stops on the first error.
    // we need something more forceful.
    fn remove_dir_all<P: AsRef<Path>>(path: P) -> std::io::Result<()> {
        let status = Command::new("rm")
            .arg("-rf")
            .arg(path.as_ref().to_string_lossy().to_string())
            .status()?;
        if status.success() {
            Ok(())
        } else {
            Err(std::io::Error::new(
                std::io::ErrorKind::Other,
                format!("Failed to remove directory: {}", path.as_ref().display()),
            ))
        }
    }

    // Helper function to create a symlink chain that eventually loops back to an earlier link
    fn setup_symlink_loop(tmp_dir: &XPath, links: &[(&str, &str)]) {
        for &(src, dst) in links {
            let src_path = tmp_dir.join(src.as_bytes());

            // Check and create parent directory for the source if necessary
            let src_parent = src_path.parent();
            if !src_parent.exists(false) {
                fs::create_dir_all(src_parent.as_path()).unwrap();
            }

            // The destination is given relative to the source
            let dst_path = XPath::from_bytes(&dst.as_bytes());

            // Avoid creating a symlink if the source already exists
            if src_path.exists(false) {
                fs::remove_file(src_path.as_path()).unwrap();
            }

            // If the destination is an absolute path or starts with '/', we strip the '/' and prefix with tmp_dir
            let full_dst_path = if dst_path.is_absolute() {
                tmp_dir.join(dst_path.strip_prefix(b"/").unwrap().as_bytes())
            } else {
                src_parent.join(dst_path.as_bytes()).into()
            };

            // Create parent directories for the destination if they don't exist
            let dst_parent = full_dst_path.parent();
            if !dst_parent.exists(false) {
                fs::create_dir_all(dst_parent.as_path()).unwrap();
            }

            // Create the symlink
            symlink(full_dst_path.as_path(), src_path.as_path())
                .expect(&format!("Unable to symlink {src_path} -> {full_dst_path}",));
        }
    }

    fn tempdir() -> Result<XPathBuf, Box<dyn std::error::Error>> {
        let tmp = tempfile::Builder::new().keep(true).tempdir_in("./")?;
        let _ = OpenOptions::new()
            .write(true)
            .create(true)
            .mode(0o600)
            .open(tmp.path().join("test"))?;
        Ok(tmp
            .path()
            .to_path_buf()
            .file_name()
            .unwrap()
            .as_bytes()
            .into())
    }

    #[test]
    fn test_canonicalize_empty_path() -> TestResult {
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b""),
                None,
                FsFlags::empty(),
                Flags::empty()
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b""),
                None,
                FsFlags::MUST_PATH,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b""),
                None,
                FsFlags::MISS_LAST,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b""),
                None,
                FsFlags::NO_FOLLOW_LAST,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b""),
                None,
                FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b""),
                None,
                FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );

        Ok(())
    }

    #[test]
    fn test_canonicalize_repetitive_root() -> TestResult {
        let (root, root_type) = canonicalize(
            Pid::this(),
            &XPath::from_bytes(b"//"),
            None,
            FsFlags::empty(),
            Flags::empty(),
        )?;
        assert_eq!(root.as_bytes(), b"/");
        assert_eq!(root_type, Some(FileType::Dir));

        Ok(())
    }

    #[test]
    fn test_canonicalize_repetitive_slashes() -> TestResult {
        let result_test = canonicalize(
            Pid::this(),
            &XPath::from_bytes(b"/etc/passwd"),
            None,
            FsFlags::empty(),
            Flags::empty(),
        )?;
        let paths = vec![
            "/etc/passwd",
            "/etc//passwd",
            "/etc///passwd",
            "//etc/passwd",
            "//etc//passwd",
            "//etc///passwd",
            "///etc/passwd",
            "///etc//passwd",
            "///etc///passwd",
        ];
        for path in &paths {
            let path = XPathBuf::from(path.to_string());
            let result = canonicalize(
                Pid::this(),
                &path.deref(),
                None,
                FsFlags::empty(),
                Flags::empty(),
            )?;
            assert_eq!(result, result_test);
        }

        Ok(())
    }

    #[test]
    fn test_canonicalize_dots_slashes() -> TestResult {
        let cwd = XPathBuf::from(std::env::current_dir()?);
        let tmp = tempdir()?;

        let path = xpath!("{tmp}//./..//{tmp}/test");
        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            Some(&cwd.deref()),
            FsFlags::empty(),
            Flags::empty(),
        );
        assert!(result.is_ok(), "{path}->{result:?}");
        let (result1, _) = result.unwrap();
        let path = xpath!("{cwd}/{tmp}//./..//{tmp}/test");
        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{path}->{result:?}");
        let (result2, _) = result.unwrap();

        assert!(!result1.is_empty(), "result:{result1}");
        assert!(!result2.is_empty(), "result:{result2}");
        assert_eq!(result1, result2);

        Ok(())
    }

    #[test]
    fn test_canonicalize_non_directory_with_slash() -> TestResult {
        let cwd = XPathBuf::from(std::env::current_dir()?.display().to_string());
        let path = tempdir()?;
        let test = xpath!("{path}/test/");

        assert_eq!(
            canonicalize(
                Pid::this(),
                &test.deref(),
                Some(&cwd.deref()),
                FsFlags::empty(),
                Flags::empty(),
            ),
            Err(Errno::ENOTDIR)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &test.deref(),
                Some(&cwd.deref()),
                FsFlags::MUST_PATH,
                Flags::empty(),
            ),
            Err(Errno::ENOTDIR)
        );
        //SAFETY: Missing is handled by read_path*.
        assert!(canonicalize(
            Pid::this(),
            &test.deref(),
            Some(&cwd.deref()),
            FsFlags::MISS_LAST,
            Flags::empty(),
        )
        .is_ok());
        assert_eq!(
            canonicalize(
                Pid::this(),
                &test.deref(),
                Some(&cwd.deref()),
                FsFlags::NO_FOLLOW_LAST,
                Flags::empty(),
            ),
            Err(Errno::ENOTDIR)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &test.deref(),
                Some(&cwd.deref()),
                FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
                Flags::empty(),
            ),
            Err(Errno::ENOTDIR)
        );
        //SAFETY: Missing is handled by read_path*.
        assert!(canonicalize(
            Pid::this(),
            &test.deref(),
            Some(&cwd.deref()),
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        )
        .is_ok());

        Ok(())
    }

    /// FIXME: The asserts return success rather than failure.
    /// Bug or feature?
    #[test]
    #[ignore]
    fn test_canonicalize_missing_directory_returns_enoent() -> TestResult {
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b"/zzz/.."),
                None,
                FsFlags::empty(),
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b"/zzz/.."),
                None,
                FsFlags::MUST_PATH,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b"/zzz/.."),
                None,
                FsFlags::NO_FOLLOW_LAST,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );
        assert_eq!(
            canonicalize(
                Pid::this(),
                &XPath::from_bytes(b"/zzz/.."),
                None,
                FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
                Flags::empty(),
            ),
            Err(Errno::ENOENT)
        );

        Ok(())
    }

    #[test]
    fn test_relative_symlink_resolution() -> TestResult {
        // Setup
        let root_test_dir = &XPath::from_bytes(b"test_root_relative_symlink_resolution");
        let deep_dir = root_test_dir.join(b"a/b/c");
        let _ = remove_dir_all(&root_test_dir);
        fs::create_dir_all(&root_test_dir.join(b"d"))?;
        fs::create_dir_all(&deep_dir)?;

        // Create a symlink in "b" that points upwards to "a"
        let rel_link = root_test_dir.join(b"a/b/rel_link");
        symlink("../..", &rel_link)?;

        // Append /proc/self/cwd to get an absolute path to our symlinked path
        let abs_link_path = &XPath::from_bytes(b"/proc/self/cwd")
            .join(root_test_dir.join(b"a/b/rel_link/d").as_bytes());

        // Call canonicalize
        let result = canonicalize(
            Pid::this(),
            &abs_link_path.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(
            result.is_ok(),
            "canonicalize:{abs_link_path} result:{result:?}",
        );
        let (resolved_path, _) = result.unwrap();

        // We expect the path to be resolved to "test_root/a/d", but we need to canonicalize it
        let expected_path = fs::canonicalize(
            &XPath::from_bytes(b"/proc/self/cwd").join(root_test_dir.join(b"d").as_bytes()),
        )?;

        // Cleanup:
        // Note, remove_dir_all cannot remove a directory
        // tree if it spots a symbolic link loop unlike
        // `rm -rf`.
        // let _ = remove_dir_all(&root_test_dir);
        Command::new("/bin/rm")
            .arg("-rf")
            .arg(&root_test_dir)
            .status()
            .expect("rm -rf tmpdir");

        assert_eq!(resolved_path, XPathBuf::from(expected_path));

        Ok(())
    }

    // FIXME: This test broke after we removed normalize()
    // The question: Is the test incorrect or is canonicalize()?
    #[ignore]
    #[test]
    fn test_complex_interplay_symlinks_dots() -> TestResult {
        // Setup
        let cwd = XPathBuf::from(Path::new("/proc/self/cwd").canonicalize()?);
        let root_test_dir = cwd.join(b"test_root_complex_interplay_symlinks_dots");
        let _ = remove_dir_all(&root_test_dir);
        fs::create_dir_all(root_test_dir.join(b"a/b/c")).unwrap();
        fs::create_dir(root_test_dir.join(b"d")).unwrap();
        fs::create_dir(root_test_dir.join(b"e")).unwrap();
        fs::create_dir(root_test_dir.join(b"x")).unwrap();

        // Create several symlinks
        symlink("./a", root_test_dir.join(b"link_to_a")).unwrap();
        symlink("e", root_test_dir.join(b"link_to_e")).unwrap();
        symlink("a/b", root_test_dir.join(b"link_to_b")).unwrap();
        symlink("../../x", root_test_dir.join(b"a/b/rel_link")).unwrap();

        let path = root_test_dir.join(b"link_to_a/../link_to_b/rel_link/../..");
        let (resolved_path, _) = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        )
        .unwrap();

        // Cleanup
        let _ = remove_dir_all(&root_test_dir);

        // Assertion
        assert_eq!(resolved_path, XPathBuf::from(root_test_dir));

        Ok(())
    }

    #[test]
    fn test_trailing_slash_handling() -> TestResult {
        // Spawn a new process which will inherit the fds.
        // Note we cannot use the current process here,
        // as XPath::check will think it's Syd and deny
        // access.
        let child = match unsafe { fork() } {
            Ok(ForkResult::Parent { child }) => child,
            Ok(ForkResult::Child) => {
                pause();
                exit(127);
            }
            Err(errno) => exit(errno as i32),
        };

        let path = XPath::from_bytes(b"/usr/");
        let (pabs, _) = canonicalize(child, &path, None, FsFlags::empty(), Flags::empty())?;
        assert_eq!(path, pabs.deref());

        let path = XPath::from_bytes(b"/proc/self/");
        let pexp = xpath!("/proc/{child}/");
        let (pabs, _) = canonicalize(child, &path, None, FsFlags::empty(), Flags::empty())?;
        assert_eq!(pabs, pexp);
        let (pabs, _) = canonicalize(child, &path, None, FsFlags::NO_FOLLOW_LAST, Flags::empty())?;
        assert_eq!(pabs, pexp);

        kill(child, Signal::SIGKILL)?;
        Ok(())
    }

    #[ignore]
    #[test]
    fn test_canonicalize_no_atime_change_normal() -> TestResult {
        let cdir = XPathBuf::from(std::env::current_dir()?);
        let base = cdir.join(tempdir()?.as_bytes());
        let path = base.join(b"file");
        fs::File::create(&path)?;

        assert_atime_unchanged(&path, || {
            canonicalize(
                Pid::this(),
                &path.deref(),
                None,
                FsFlags::empty(),
                Flags::empty(),
            )
        });

        let _ = remove_dir_all(&base);
        Ok(())
    }

    #[ignore]
    #[test]
    fn test_canonicalize_no_atime_change_existing() -> TestResult {
        let cdir = XPathBuf::from(std::env::current_dir()?);
        let base = cdir.join(&tempdir()?.as_bytes());
        let path = base.join(b"file");
        fs::File::create(&path)?;

        assert_atime_unchanged(&path, || {
            canonicalize(
                Pid::this(),
                &path.deref(),
                None,
                FsFlags::MUST_PATH,
                Flags::empty(),
            )
        });

        let _ = remove_dir_all(&base);
        Ok(())
    }

    #[test]
    fn test_canonicalize_symlink_loop() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");
        let dir_path = XPathBuf::from(tmp_dir.path().to_path_buf());

        // Create a symlink loop: link_a -> link_b -> link_a
        let mut link_a = dir_path.join(b"link_a");
        let mut link_b = dir_path.join(b"link_b");
        symlink(&link_b, &link_a).expect("Failed to create symlink a");
        symlink(&link_a, &link_b).expect("Failed to create symlink b");

        // Now check that canonicalize detects the loop correctly
        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        // Add a trailing slash and retest.
        link_a.push(b"");

        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &link_a.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        // Add a trailing slash and retest.
        link_b.push(b"");

        let result = canonicalize(
            Pid::this(),
            &link_b.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &link_b.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));
    }

    #[test]
    fn test_canonicalize_nonexistent_final_component() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");
        let dir_path = XPathBuf::from(tmp_dir.path().to_path_buf());

        // Create a valid symlink to a non-existent final component
        let mut valid_link = dir_path.join(b"valid_link");
        let nonexistent_target = dir_path.join(b"nonexistent");
        symlink(&nonexistent_target, &valid_link)
            .expect("Failed to create symlink to non-existent target");

        // Now check that canonicalize handles the non-existent final component correctly
        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ENOENT));

        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        // FIXME: Why oh why?
        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        // should be: assert_eq!(result, Err(Errno::EEXIST));

        // Add a trailing slash and retest.
        valid_link.push(b"");

        // FIXME: Why oh why?
        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        // should be: assert_eq!(result, Err(Errno::EEXIST));

        // FIXME: Why oh why?
        let result = canonicalize(
            Pid::this(),
            &valid_link.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        // should be: assert_eq!(result, Err(Errno::EEXIST));
    }

    #[test]
    fn test_canonicalize_self_referential_symlink() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");
        let dir_path = XPathBuf::from(tmp_dir.path().to_path_buf());
        let mut symlink_path = dir_path.join(b"self_link");
        symlink(symlink_path.as_path(), symlink_path.as_path())
            .expect("Failed to create self-referential symlink");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        // Add a trailing slash and retest.
        symlink_path.push(b"");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));
    }

    #[test]
    fn test_canonicalize_broken_symlink() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");
        let dir_path = XPathBuf::from(tmp_dir.path().to_path_buf());
        let mut broken_link = dir_path.join(b"broken_link");
        let nonexistent_target = dir_path.join(b"nonexistent_target");
        symlink(nonexistent_target.as_path(), broken_link.as_path())
            .expect("Failed to create broken symlink");

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        // FIXME: assert_eq!(result, Err(Errno::ENOENT));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ENOENT));

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        // Add a trailing slash and retest.
        broken_link.push(b"");

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &broken_link.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
    }

    #[test]
    fn test_canonicalize_symlink_to_directory() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");
        let tmp_path = XPathBuf::from(tmp_dir.path().to_path_buf());
        let dir = tmp_path.join(b"dir");
        fs::create_dir(&dir).expect("Failed to create directory");

        let symlink_path = tmp_path.join(b"dir_link");
        symlink(dir.as_path(), symlink_path.as_path())
            .expect("Failed to create symlink to directory");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_symlink(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_dir(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_symlink(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_dir(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &symlink_path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");
    }

    #[test]
    fn test_canonicalize_symlink_chain() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");
        let dir_path = XPathBuf::from(tmp_dir.path().to_path_buf());
        let link1 = dir_path.join(b"link1");
        let link2 = dir_path.join(b"link2");
        let link3 = dir_path.join(b"link3");
        let file = dir_path.join(b"file");
        fs::write(file.as_path(), "content").expect("Failed to write file");

        // Create a chain of symlinks: link1 -> link2 -> link3 -> file
        symlink(link2.as_path(), link1.as_path()).expect("Failed to create link1");
        symlink(link3.as_path(), link2.as_path()).expect("Failed to create link2");
        symlink(file.as_path(), link3.as_path()).expect("Failed to create link3");

        let result = canonicalize(
            Pid::this(),
            &link1.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_symlink(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link1.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_file(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link1.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_symlink(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link1.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_file(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link1.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &link1.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");
    }

    #[test]
    fn test_canonicalize_complex_symlink_loop_with_intermediate_components() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");
        let dir_path = XPathBuf::from(tmp_dir.path().to_path_buf());

        // Setting up a complex symlink scenario
        setup_symlink_loop(
            &dir_path.deref(),
            &[("a", "b/c"), ("b/c", "d"), ("b/d", "../e"), ("e", "f/../a")],
        );

        let mut path = dir_path.join(b"a");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_symlink(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{result:?}");
        assert!(result.clone().unwrap().0.is_symlink(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        // Add a trailing slash and retest.
        path.push(b"");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        // Add a final component and retest.
        path.push(b"foo");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));
    }

    #[test]
    fn test_canonicalize_symlinks_with_dot_and_dotdot_components() {
        let tmp_dir = tempfile::Builder::new()
            .keep(true)
            .tempdir()
            .expect("Failed to create temp dir");

        // Create a complex directory structure with dots and symlinks
        fs::create_dir_all(tmp_dir.path().join("b")).expect("Failed to create directory b");
        symlink("b", tmp_dir.path().join("a")).expect("Failed to create symlink a -> b");
        symlink("..///e", tmp_dir.path().join("b").join("d"))
            .expect("Failed to create symlink b/d -> ../e");
        symlink("b/.///./d", tmp_dir.path().join("e")).expect("Failed to create symlink e -> b/d");

        let mut path = XPathBuf::from(tmp_dir.path().join("a").join(".").join("d"));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{path}->{result:?}");
        assert!(
            result.clone().unwrap().0.exists(false),
            "{path}->{result:?}"
        );
        assert!(
            !result.clone().unwrap().0.exists(true),
            "{path}->{result:?}"
        );

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::empty(),
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert!(result.is_ok(), "{path}->{result:?}");
        assert!(
            result.clone().unwrap().0.exists(false),
            "{path}->{result:?}"
        );
        assert!(
            !result.clone().unwrap().0.exists(true),
            "{path}->{result:?}"
        );

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MUST_PATH,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        // Add a trailing slash and retest.
        path.push(b"");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        // Add a final component and retest.
        path.push(b"foo");

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::NO_FOLLOW_LAST | FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));

        let result = canonicalize(
            Pid::this(),
            &path.deref(),
            None,
            FsFlags::MISS_LAST,
            Flags::empty(),
        );
        assert_eq!(result, Err(Errno::ELOOP));
    }

    #[test]
    fn test_canonicalize_proc_self() -> TestResult {
        let path = canonicalize(
            Pid::this(),
            &XPath::from_bytes(b"/proc/self"),
            None,
            FsFlags::NO_FOLLOW_LAST,
            Flags::empty(),
        )?
        .0;
        assert_eq!(path.deref(), XPath::from_bytes(b"/proc/self"));

        Ok(())
    }

    #[test]
    fn test_canon_glob_std() -> TestResult {
        // Note we cannot assume the FDs {0,1,2} are open here.
        let fd = open("/dev/null", OFlag::O_RDONLY, Mode::empty())?;
        // SAFETY: open returns a valid FD.
        let fd = unsafe { OwnedFd::from_raw_fd(fd) };
        let fd = xpath!("/proc/self/fd/{}", fd.as_raw_fd());
        let pid = Pid::this();

        let result = canonicalize(pid, &fd.deref(), None, FsFlags::empty(), Flags::empty());
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(pid, &fd.deref(), None, FsFlags::MUST_PATH, Flags::empty());
        assert!(result.is_ok(), "{result:?}");

        let result = canonicalize(pid, &fd.deref(), None, FsFlags::MISS_LAST, Flags::empty());
        //SAFETY: Missing is handled by read_path*.
        //assert_eq!(result, Err(Errno::EEXIST));
        assert!(result.is_ok(), "{result:?}");

        Ok(())
    }

    #[test]
    fn test_canon_glob_pipe() -> TestResult {
        // Create a socket pair.
        let (read_end, write_end) = UnixStream::pair()?;

        // Spawn a new process which will inherit the fds.
        // Note we cannot use the current process here,
        // as XPath::check will think it's Syd and deny
        // access.
        let child = match unsafe { fork() } {
            Ok(ForkResult::Parent { child }) => child,
            Ok(ForkResult::Child) => {
                pause();
                exit(127);
            }
            Err(errno) => exit(errno as i32),
        };

        let fd = read_end.as_raw_fd();
        let path = xpath!("/proc/{child}/fd/{fd}");
        let result = canonicalize(child, &path.deref(), None, FsFlags::empty(), Flags::empty());
        assert!(result.is_ok(), "{path}->{result:?}");

        let fd = write_end.as_raw_fd();
        let path = xpath!("/proc/{child}/fd/{fd}");
        let result = canonicalize(child, &path.deref(), None, FsFlags::empty(), Flags::empty());
        assert!(result.is_ok(), "{path}->{result:?}");

        kill(child, Signal::SIGKILL)?;
        Ok(())
    }
}
