// Syd: rock-solid application kernel
// src/syd-bit.rs: Utility to flip bits in a file
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0

use std::{
    env,
    fs::OpenOptions,
    io::{Read, Seek, SeekFrom, Write},
    process::ExitCode,
};

use getargs::{Opt, Options};
use hex::DisplayHex;
use syd::{err::SydResult, fs::fillrandom};

// Set global allocator to mimalloc.
#[global_allocator]
static GLOBAL: mimalloc2_rust::GlobalMiMalloc = mimalloc2_rust::GlobalMiMalloc;

fn main() -> SydResult<ExitCode> {
    syd::set_sigpipe_dfl()?;

    // Configure mimalloc.
    syd::configure_mimalloc();

    let args = env::args().skip(1).collect::<Vec<_>>();
    let args: &'static Vec<String> = Box::leak(Box::new(args));
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut index: Option<usize> = None;
    let mut random = false;

    while let Some(opt) = opts.next_opt()? {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('i') => {
                index = if let Ok(value) = opts.value() {
                    Some(value.parse::<usize>()?)
                } else {
                    eprintln!("Error: -i requires an argument.");
                    help();
                    return Ok(ExitCode::FAILURE);
                };
            }
            Opt::Short('r') => {
                random = true;
            }
            Opt::Short(opt) => {
                eprintln!("Unknown option: -{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
            Opt::Long(opt) => {
                eprintln!("Unknown option: --{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    if let Some(file_name) = opts.positionals().next() {
        match (index, random) {
            (Some(idx), false) => flip_bit_in_file(file_name, idx)?,
            (None, true) => flip_random_bit_in_file(file_name)?,
            _ => {
                eprintln!("Exactly one of -i <index> or -r must be given!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    } else {
        eprintln!("File name not specified!");
        help();
        return Ok(ExitCode::FAILURE);
    }

    Ok(ExitCode::SUCCESS)
}

fn flip_bit_in_file(file_name: &str, bit_index: usize) -> SydResult<()> {
    eprintln!("syd-bit: opening file {file_name}...");
    let mut file = OpenOptions::new().read(true).write(true).open(file_name)?;

    // Calculate the byte index from the bit index.
    let byte_index = bit_index / 8;
    let bit_in_byte = bit_index % 8;

    // Seek to the byte containing the bit.
    eprintln!("syd-bit: seeking to byte index {byte_index}...");
    file.seek(SeekFrom::Start(byte_index as u64))?;

    // Read the single byte.
    let mut byte = [0u8; 1];
    file.read_exact(&mut byte)?;
    eprintln!(
        "syd-bit: read byte 0x{} at index {byte_index}.",
        byte.to_upper_hex_string()
    );

    // Flip the specified bit in the byte
    byte[0] ^= 1 << bit_in_byte;
    eprintln!(
        "syd-bit: flipped bit {bit_in_byte} resulting in 0x{}.",
        byte.to_upper_hex_string()
    );

    // Move the file pointer back to the byte to overwrite it
    eprintln!("syd-bit: moving back to file offset {byte_index}.");
    file.seek(SeekFrom::Start(byte_index as u64))?;

    // Write the modified byte back to the file
    eprintln!(
        "syd-bit: writing byte 0x{} at index {byte_index}...",
        byte.to_upper_hex_string()
    );
    file.write_all(&byte)?;

    eprintln!("syd-bit: flipped bit:{bit_index} of byte:{byte_index}.");
    Ok(())
}

fn flip_random_bit_in_file(file_name: &str) -> SydResult<()> {
    let mut file = OpenOptions::new().read(true).write(true).open(file_name)?;

    let file_len = file.seek(SeekFrom::End(0))?; // Get file length
    let total_bits = file_len * 8;

    let mut rng_buf = [0u8; std::mem::size_of::<usize>()];
    fillrandom(&mut rng_buf)?; // secure random fill
    let bit_index = usize::from_ne_bytes(rng_buf) % total_bits as usize;

    flip_bit_in_file(file_name, bit_index)
}

fn help() {
    println!("Usage: syd-bit [-h] -i <idx> <file> | -r <file>");
    println!("Utility to flip bits in files");
    println!("  -i <idx>    Flip the bit at index <idx> in the file");
    println!("  -r          Flip a random bit in the file");
}
