//
// Syd: rock-solid application kernel
// src/syd-dns.rs: Resolve hostname into IPs using system DNS resolver
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0

use std::process::ExitCode;

use getargs::{Opt, Options};
use nix::{
    errno::Errno,
    libc::{AF_INET, AF_INET6},
};
use syd::{
    dns::{resolve_host, resolve_rand},
    err::SydResult,
};

// Set global allocator to mimalloc.
#[global_allocator]
static GLOBAL: mimalloc2_rust::GlobalMiMalloc = mimalloc2_rust::GlobalMiMalloc;

fn main() -> SydResult<ExitCode> {
    syd::set_sigpipe_dfl()?;

    // Configure mimalloc.
    syd::configure_mimalloc();

    let args = std::env::args().skip(1).collect::<Vec<_>>();
    let args: &'static Vec<String> = Box::leak(Box::new(args));
    let mut opts = Options::new(args.iter().map(String::as_str));

    let mut opt_ipv4 = false;
    let mut opt_ipv6 = false;
    let mut opt_rand = false;

    while let Some(opt) = opts.next_opt()? {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('r') => opt_rand = true,
            Opt::Short('4') => opt_ipv4 = true,
            Opt::Short('6') => opt_ipv6 = true,
            Opt::Short(opt) => {
                eprintln!("Unknown option: -{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
            Opt::Long(opt) => {
                eprintln!("Unknown option: --{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    let family = if opt_ipv4 && opt_ipv6 {
        eprintln!("At most one of -4, -6 must be given!");
        return Err(Errno::EINVAL.into());
    } else if opt_ipv4 {
        Some(AF_INET)
    } else if opt_ipv6 {
        Some(AF_INET6)
    } else {
        None
    };

    let name = if let Some(name) = opts.positionals().next() {
        name
    } else {
        help();
        return Err(Errno::ENOENT.into());
    };

    if opt_rand {
        let addr = resolve_rand(name, family)?;
        println!("{addr}");
    } else {
        for addr in resolve_host(name, family)? {
            println!("{addr}");
        }
    }

    Ok(ExitCode::SUCCESS)
}

fn help() {
    println!("Usage: syd-dns [-hr46] hostname");
    println!("Resolve hostname into IPs using system DNS resolver.");
    println!("If -4 is given, print only IPv4 addresses.");
    println!("If -6 is given, print only IPv6 addresses.");
    println!("If -r is given, print a random IP.");
}
