//
// Syd: rock-solid application kernel
// src/syd-hex.rs: Hexadecimal encode/decode standard input.
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0

use std::{
    env,
    fs::File,
    io::{BufRead, BufReader, Write},
    process::ExitCode,
};

use getargs::{Opt, Options};
use hex::{DisplayHex, FromHex};
use syd::err::SydResult;

// Set global allocator to mimalloc.
#[global_allocator]
static GLOBAL: mimalloc2_rust::GlobalMiMalloc = mimalloc2_rust::GlobalMiMalloc;

fn main() -> SydResult<ExitCode> {
    syd::set_sigpipe_dfl()?;

    // Configure mimalloc.
    syd::configure_mimalloc();

    let args = env::args().skip(1).collect::<Vec<_>>();
    let args: &'static Vec<String> = Box::leak(Box::new(args));
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut encode = true;
    let mut climit: Option<usize> = None;

    while let Some(opt) = opts.next_opt()? {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('d') => {
                encode = false;
            }
            Opt::Short('e') => {
                encode = true;
            }
            Opt::Short('l') => {
                climit = match opts.value() {
                    Ok(val) => match parse_size::Config::new().with_binary().parse_size(val) {
                        Ok(val) => match usize::try_from(val) {
                            Ok(val) => Some(val),
                            Err(err) => {
                                eprintln!("invalid limit given to -l: {err}");
                                return Ok(ExitCode::FAILURE);
                            }
                        },
                        Err(err) => {
                            eprintln!("invalid limit given to -l: {err}");
                            return Ok(ExitCode::FAILURE);
                        }
                    },
                    Err(_) => {
                        eprintln!("-l requires an argument!");
                        return Ok(ExitCode::FAILURE);
                    }
                };
            }
            Opt::Short(opt) => {
                eprintln!("Unknown option: -{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
            Opt::Long(opt) => {
                eprintln!("Unknown option: --{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    // Lock stdin for efficient reading
    #[allow(clippy::disallowed_methods)]
    let mut input: Box<dyn BufRead> = match opts.positionals().next() {
        None | Some("-") => Box::new(std::io::stdin().lock()),
        Some(path) => Box::new(BufReader::new(File::open(path)?)),
    };

    // Process the input in chunks to avoid loading
    // it all into memory at once.
    let mut nwrite: usize = 0;
    let mut buffer = [0; 64 * 1024]; // 64KB buffer size.
    while let Ok(count) = input.read(&mut buffer[..]) {
        let buffer = if count == 0 {
            break; // End of input.
        } else if let Some(lim) = climit {
            let buffer = if nwrite.checked_add(count).map(|c| c >= lim).unwrap_or(true) {
                let offset = match lim.checked_sub(nwrite) {
                    Some(0) | None => break, // Limit reached.
                    Some(n) => n,
                };
                &buffer[..offset]
            } else {
                &buffer[..count]
            };
            nwrite = nwrite.checked_add(count).unwrap_or(usize::MAX);
            buffer
        } else {
            &buffer[..count]
        };
        if encode {
            // Hex-encode and write the chunk
            let encoded = buffer.to_lower_hex_string();
            print!("{encoded}");
        } else {
            // Hex-decode and write the chunk
            #[allow(clippy::disallowed_methods)]
            match Vec::from_hex(std::str::from_utf8(buffer)?) {
                Ok(decoded) => {
                    std::io::stdout().write_all(&decoded)?;
                }
                Err(error) => {
                    eprintln!("Error decoding hex: {error}");
                    return Ok(ExitCode::FAILURE);
                }
            }
        }
    }

    Ok(ExitCode::SUCCESS)
}

fn help() {
    println!("Usage: syd-hex [-hdel] <file|->");
    println!("Given a file, hex-encode and print.");
    println!("Given no positional arguments, hex-encode standard input.");
    println!("Use -d to hex-decode rather than hex-encode.");
    println!("Use -l <human-size> to exit after size bytes are read.");
}
