//
// Syd: rock-solid application kernel
// src/syd-info.rs: Print system information.
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0

use std::{mem::MaybeUninit, process::ExitCode};

use getargs::{Opt, Options};
use nix::errno::Errno;
use serde_json::json;
use syd::err::SydResult;

// Set global allocator to mimalloc.
#[global_allocator]
static GLOBAL: mimalloc2_rust::GlobalMiMalloc = mimalloc2_rust::GlobalMiMalloc;

fn main() -> SydResult<ExitCode> {
    syd::set_sigpipe_dfl()?;

    // Configure mimalloc.
    syd::configure_mimalloc();

    let args = std::env::args().skip(1).collect::<Vec<_>>();
    let args: &'static Vec<String> = Box::leak(Box::new(args));
    let mut opts = Options::new(args.iter().map(String::as_str));

    while let Some(opt) = opts.next_opt()? {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short(opt) => {
                eprintln!("Unknown option: -{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
            Opt::Long(opt) => {
                eprintln!("Unknown option: --{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    let mut info = MaybeUninit::<nix::libc::sysinfo>::uninit();
    // SAFETY: In libc we trust.
    Errno::result(unsafe { nix::libc::sysinfo(info.as_mut_ptr()) })?;
    // SAFETY: sysinfo() has initialized `info` if it succeeded.
    let info = unsafe { info.assume_init() };

    #[allow(clippy::disallowed_methods)]
    let info = json!({
        "uptime": info.uptime,
        "loads": info.loads,
        "totalram": info.totalram,
        "freeram": info.freeram,
        "sharedram": info.sharedram,
        "bufferram": info.bufferram,
        "totalswap": info.totalswap,
        "freeswap": info.freeswap,
        "procs": info.procs,
        "totalhigh": info.totalhigh,
        "freehigh": info.freehigh,
        "mem_unit": info.mem_unit,
    });

    #[allow(clippy::disallowed_methods)]
    let info = serde_json::to_string_pretty(&info).unwrap();
    println!("{info}");

    Ok(ExitCode::SUCCESS)
}

fn help() {
    println!("Usage: syd-info [-h]");
    println!("Print system information.");
}
