//
// Syd: rock-solid application kernel
// src/syd-key.rs: Utility to generate AES-CTR Key and IV
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0

use std::{env, io::Read, process::ExitCode};

use getargs::{Opt, Options};
use syd::{err::SydResult, hash::Key};

// Set global allocator to mimalloc.
#[global_allocator]
static GLOBAL: mimalloc2_rust::GlobalMiMalloc = mimalloc2_rust::GlobalMiMalloc;

fn main() -> SydResult<ExitCode> {
    syd::set_sigpipe_dfl()?;

    // Configure mimalloc.
    syd::configure_mimalloc();

    let args = env::args().skip(1).collect::<Vec<_>>();
    let args: &'static Vec<String> = Box::leak(Box::new(args));
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut tag_val: Option<&str> = None;

    #[allow(clippy::never_loop)]
    while let Some(opt) = opts.next_opt()? {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('t') => {
                tag_val = opts.value().ok();
            }
            Opt::Short(opt) => {
                eprintln!("Unknown option: -{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
            Opt::Long(opt) => {
                eprintln!("Unknown option: --{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    let key = if let Some(info) = tag_val {
        // Read the key from stdin.
        let mut input = Vec::new();
        std::io::stdin().read_to_end(&mut input)?;
        // Remove any whitespace from the input.
        input.retain(|&byte| !byte.is_ascii_whitespace());
        let key = Key::from_hex(&input)?;
        // Derive the key using the given info.
        key.derive(None, info.as_bytes())
    } else {
        Key::random()?
    };

    println!("{}", key.as_hex());
    Ok(ExitCode::SUCCESS)
}

fn help() {
    println!("Usage: syd-key [-h] -t <tag-str>");
    println!("Utility to generate 256-bit AES-CTR key");
    println!("  -h        Print this help message and exit");
    println!("  -t <tag>  Read key-hex from standard input, derive a new key with HKDF using the given information tag");
}
