//
// Syd: rock-solid application kernel
// src/syd-lock.rs: Run a command under Landlock
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0

use std::{
    ops::RangeInclusive,
    os::unix::process::CommandExt,
    process::{Command, ExitCode},
};

use getargs::{Opt, Options};
use nix::errno::Errno;
use syd::{
    err::{SydError, SydResult},
    landlock::{RulesetStatus, ABI},
    lock_enabled,
    path::XPathBuf,
    SYD_LOCK_VEC,
};

// Set global allocator to mimalloc.
#[global_allocator]
static GLOBAL: mimalloc2_rust::GlobalMiMalloc = mimalloc2_rust::GlobalMiMalloc;

fn main() -> SydResult<ExitCode> {
    syd::set_sigpipe_dfl()?;

    // Configure mimalloc.
    syd::configure_mimalloc();

    let args = std::env::args().skip(1).collect::<Vec<_>>();
    let args: &'static Vec<String> = Box::leak(Box::new(args));
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut abick = false;
    let mut check = false;
    let mut verbose = false;
    let mut path_ro = vec![];
    let mut path_rw = vec![];
    let mut port_bind = vec![];
    let mut port_conn = vec![];

    while let Some(opt) = opts.next_opt()? {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('A') => {
                abick = true;
            }
            Opt::Short('V') => {
                check = true;
            }
            Opt::Short('v') => {
                verbose = true;
            }
            Opt::Short('r') => match opts.value() {
                Ok(path) if !path.starts_with('/') => {
                    eprintln!("-r requires an absolute path as argument!");
                    return Ok(ExitCode::FAILURE);
                }
                Ok(path) => path_ro.push(XPathBuf::from(path)),
                Err(_) => {
                    eprintln!("-r requires an argument!");
                    return Ok(ExitCode::FAILURE);
                }
            },
            Opt::Short('w') => match opts.value() {
                Ok(path) if !path.starts_with('/') => {
                    eprintln!("-w requires an absolute path as argument!");
                    return Ok(ExitCode::FAILURE);
                }
                Ok(path) => path_rw.push(XPathBuf::from(path)),
                Err(_) => {
                    eprintln!("-w requires an argument!");
                    return Ok(ExitCode::FAILURE);
                }
            },
            Opt::Short('b') => match opts.value() {
                Ok(port) => {
                    let parts: Vec<&str> = port.splitn(2, '-').collect();
                    let port_range: RangeInclusive<u16> = if parts.len() == 2 {
                        parts[0].parse().or::<SydError>(Err(Errno::EINVAL.into()))?
                            ..=parts[1].parse().or::<SydError>(Err(Errno::EINVAL.into()))?
                    } else {
                        let p = parts[0].parse().or::<SydError>(Err(Errno::EINVAL.into()))?;
                        p..=p
                    };
                    port_bind.push(port_range);
                }
                Err(_) => {
                    eprintln!("-b requires an argument!");
                    return Ok(ExitCode::FAILURE);
                }
            },
            Opt::Short('c') => match opts.value() {
                Ok(port) => {
                    let parts: Vec<&str> = port.splitn(2, '-').collect();
                    let port_range: RangeInclusive<u16> = if parts.len() == 2 {
                        parts[0].parse().or::<SydError>(Err(Errno::EINVAL.into()))?
                            ..=parts[1].parse().or::<SydError>(Err(Errno::EINVAL.into()))?
                    } else {
                        let p = parts[0].parse().or::<SydError>(Err(Errno::EINVAL.into()))?;
                        p..=p
                    };
                    port_conn.push(port_range);
                }
                Err(_) => {
                    eprintln!("-b requires an argument!");
                    return Ok(ExitCode::FAILURE);
                }
            },
            Opt::Short(opt) => {
                eprintln!("Unknown option: -{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
            Opt::Long(opt) => {
                eprintln!("Unknown option: --{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    if abick && check {
        eprintln!("-A and -V are mutually exclusive!");
        return Err(Errno::EINVAL.into());
    }

    if abick {
        let mut abi = ABI::Unsupported;
        let mut state = 127; // not supported?
        for my_abi in SYD_LOCK_VEC {
            let abi_state = lock_enabled(*my_abi);
            if state > abi_state {
                abi = *my_abi;
                state = abi_state;
                if state == 0 {
                    break;
                }
            }
        }
        let abi = abi as i32 as u8;
        print!("{abi}");
        return Ok(ExitCode::from(abi));
    } else if check {
        let mut abi = ABI::Unsupported;
        let mut state = 127; // not supported?
        for my_abi in SYD_LOCK_VEC {
            let abi_state = lock_enabled(*my_abi);
            if state > abi_state {
                abi = *my_abi;
                state = abi_state;
                if state == 0 {
                    break;
                }
            }
        }

        let state_verb = match state {
            0 => "fully enforced",
            1 => "partially enforced",
            2 => "not enforced",
            _ => "unsupported",
        };
        println!("Landlock ABI {} is {state_verb}.", abi as i32);
        return Ok(ExitCode::from(state));
    }

    // Prepare the command to execute.
    let positionals: Vec<&str> = opts.positionals().collect();
    if positionals.is_empty() {
        help();
        return Ok(ExitCode::FAILURE);
    }

    // Set up Landlock sandbox.
    macro_rules! vprintln {
        ($($arg:tt)*) => {
            if verbose {
                eprintln!($($arg)*);
            }
        };
    }
    match syd::landlock_operation(&path_ro, &path_rw, &port_bind, &port_conn, true, true) {
        Ok(status) => match status.ruleset {
            RulesetStatus::FullyEnforced => vprintln!("syd-lock: Landlock is fully enforced."),
            RulesetStatus::PartiallyEnforced => {
                vprintln!("syd-lock: Landlock is partially enforced.")
            }
            RulesetStatus::NotEnforced => {
                eprintln!("syd-lock: Landlock is not enforced!");
                return Ok(ExitCode::FAILURE);
            }
        },
        Err(error) => {
            eprintln!("syd-lock: Landlock unsupported: {error}");
            return Ok(ExitCode::FAILURE);
        }
    };

    // Execute command.
    #[allow(clippy::disallowed_methods)]
    let (command, args) = positionals.split_first().expect("oops");
    Ok(ExitCode::from(
        127 + Command::new(command)
            .args(args)
            .exec()
            .raw_os_error()
            .unwrap_or(0) as u8,
    ))
}

fn help() {
    println!("Usage: syd-lock [-hvAV] [-r path]... [-w path]... [-b port]... [-c port]... {{command [args...]}}");
    println!("Run a command under Landlock.");
    println!("Use -r to specify a read-only path, may be repeated.");
    println!("Use -w to specify a read-write path, may be repeated.");
    println!("Use -b to specify a bind port, may be repeated.");
    println!("Use -c to specify a connect port, may be repeated.");
    println!("Use -v to increase verbosity.");
    println!("Use -A to exit with Landlock ABI version, rather than running a command.");
    println!("Use -V to check for Landlock support, rather than running a command.");
}
