//
// Syd: rock-solid application kernel
// src/syd-pds.rs: Run a command with parent death signal set
//
// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
//
// SPDX-License-Identifier: GPL-3.0

use std::{
    os::unix::process::CommandExt,
    process::{Command, ExitCode},
    str::FromStr,
};

use getargs::{Opt, Options};
use nix::{errno::Errno, sys::signal::Signal};
use syd::err::SydResult;

// Set global allocator to mimalloc.
#[global_allocator]
static GLOBAL: mimalloc2_rust::GlobalMiMalloc = mimalloc2_rust::GlobalMiMalloc;

fn main() -> SydResult<ExitCode> {
    syd::set_sigpipe_dfl()?;

    // Configure mimalloc.
    syd::configure_mimalloc();

    let args = std::env::args().skip(1).collect::<Vec<_>>();
    let args: &'static Vec<String> = Box::leak(Box::new(args));
    let mut opts = Options::new(args.iter().map(String::as_str));
    let mut sig = Signal::SIGKILL;

    while let Some(opt) = opts.next_opt()? {
        match opt {
            Opt::Short('h') => {
                help();
                return Ok(ExitCode::SUCCESS);
            }
            Opt::Short('s') => match opts.value() {
                Ok(value) => match value.parse::<i32>() {
                    Ok(value) => match Signal::try_from(value) {
                        Ok(value) => sig = value,
                        Err(_) => {
                            eprintln!("Invalid signal specified with -s!");
                            return Err(Errno::EINVAL.into());
                        }
                    },
                    Err(_) => {
                        let mut value = value.to_ascii_uppercase();
                        if !value.starts_with("SIG") {
                            value = format!("SIG{value}");
                        }
                        match Signal::from_str(&value) {
                            Ok(value) => sig = value,
                            Err(_) => {
                                eprintln!("Invalid signal specified with -s!");
                                return Err(Errno::EINVAL.into());
                            }
                        }
                    }
                },
                Err(_) => {
                    eprintln!("-s requires an argument!");
                    return Ok(ExitCode::FAILURE);
                }
            },
            Opt::Short(opt) => {
                eprintln!("Unknown option: -{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
            Opt::Long(opt) => {
                eprintln!("Unknown option: --{opt}!");
                help();
                return Ok(ExitCode::FAILURE);
            }
        }
    }

    // Set parent death signal to the specified signal or SIGKILL.
    Errno::result(unsafe {
        nix::libc::prctl(
            nix::libc::PR_SET_PDEATHSIG,
            sig as nix::libc::c_ulong,
            0,
            0,
            0,
        )
    })?;

    // Prepare the command to execute.
    let positionals: Vec<&str> = opts.positionals().collect();
    if positionals.is_empty() {
        help();
        return Ok(ExitCode::FAILURE);
    }

    // Execute command.
    #[allow(clippy::disallowed_methods)]
    let (command, args) = positionals.split_first().expect("oops");
    Ok(ExitCode::from(
        127 + Command::new(command)
            .args(args)
            .exec()
            .raw_os_error()
            .unwrap_or(0) as u8,
    ))
}

fn help() {
    println!("Usage: syd-pds [-h] [-s signal] {{command [args...]}}");
    println!("Run a command with parent death signal set.");
    println!("Use -s to specify a signal, defaults to SIGKILL.");
}
